# -*- coding: utf-8 -*-
#
# Copyright (c) ZeroC, Inc. All rights reserved.
#
#
# Ice version 3.7.6
#
# <auto-generated>
#
# Generated from file `ObjectAdapter.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import Ice.CommunicatorF_ice
import Ice.ServantLocatorF_ice
import Ice.Locator_ice
import Ice.FacetMap_ice
import Ice.Endpoint_ice

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Start of module Ice
__name__ = 'Ice'

if 'ObjectAdapter' not in _M_Ice.__dict__:
    _M_Ice.ObjectAdapter = Ice.createTempClass()
    class ObjectAdapter(object):
        """
        The object adapter provides an up-call interface from the Ice
        run time to the implementation of Ice objects.
        The object adapter is responsible for receiving requests
        from endpoints, and for mapping between servants, identities, and
        proxies.
        """
        def __init__(self):
            if Ice.getType(self) == _M_Ice.ObjectAdapter:
                raise RuntimeError('Ice.ObjectAdapter is an abstract class')

        def getName(self):
            """
            Get the name of this object adapter.
            Returns: This object adapter's name.
            """
            raise NotImplementedError("method 'getName' not implemented")

        def getCommunicator(self):
            """
            Get the communicator this object adapter belongs to.
            Returns: This object adapter's communicator.
            """
            raise NotImplementedError("method 'getCommunicator' not implemented")

        def activate(self):
            """
            Activate all endpoints that belong to this object adapter.
            After activation, the object adapter can dispatch requests
            received through its endpoints.
            """
            raise NotImplementedError("method 'activate' not implemented")

        def hold(self):
            """
            Temporarily hold receiving and dispatching requests. The object
            adapter can be reactivated with the activate operation.
             Holding is not immediate, i.e., after hold
            returns, the object adapter might still be active for some
            time. You can use waitForHold to wait until holding is
            complete.
            """
            raise NotImplementedError("method 'hold' not implemented")

        def waitForHold(self):
            """
            Wait until the object adapter holds requests. Calling hold
            initiates holding of requests, and waitForHold only returns
            when holding of requests has been completed.
            """
            raise NotImplementedError("method 'waitForHold' not implemented")

        def deactivate(self):
            """
            Deactivate all endpoints that belong to this object adapter.
            After deactivation, the object adapter stops receiving
            requests through its endpoints. Object adapters that have been
            deactivated must not be reactivated again, and cannot be used
            otherwise. Attempts to use a deactivated object adapter raise
            ObjectAdapterDeactivatedException however, attempts to
            deactivate an already deactivated object adapter are
            ignored and do nothing. Once deactivated, it is possible to
            destroy the adapter to clean up resources and then create and
            activate a new adapter with the same name.
             After deactivate returns, no new requests
            are processed by the object adapter. However, requests that
            have been started before deactivate was called might
            still be active. You can use waitForDeactivate to wait
            for the completion of all requests for this object adapter.
            """
            raise NotImplementedError("method 'deactivate' not implemented")

        def waitForDeactivate(self):
            """
            Wait until the object adapter has deactivated. Calling
            deactivate initiates object adapter deactivation, and
            waitForDeactivate only returns when deactivation has
            been completed.
            """
            raise NotImplementedError("method 'waitForDeactivate' not implemented")

        def isDeactivated(self):
            """
            Check whether object adapter has been deactivated.
            Returns: Whether adapter has been deactivated.
            """
            raise NotImplementedError("method 'isDeactivated' not implemented")

        def destroy(self):
            """
            Destroys the object adapter and cleans up all resources held by
            the object adapter. If the object adapter has not yet been
            deactivated, destroy implicitly initiates the deactivation
            and waits for it to finish. Subsequent calls to destroy are
            ignored. Once destroy has returned, it is possible to create
            another object adapter with the same name.
            """
            raise NotImplementedError("method 'destroy' not implemented")

        def add(self, servant, id):
            """
            Add a servant to this object adapter's Active Servant Map. Note
            that one servant can implement several Ice objects by registering
            the servant with multiple identities. Adding a servant with an
            identity that is in the map already throws AlreadyRegisteredException.
            Arguments:
            servant -- The servant to add.
            id -- The identity of the Ice object that is implemented by the servant.
            Returns: A proxy that matches the given identity and this object adapter.
            """
            raise NotImplementedError("method 'add' not implemented")

        def addFacet(self, servant, id, facet):
            """
            Like add, but with a facet. Calling add(servant, id)
            is equivalent to calling addFacet with an empty facet.
            Arguments:
            servant -- The servant to add.
            id -- The identity of the Ice object that is implemented by the servant.
            facet -- The facet. An empty facet means the default facet.
            Returns: A proxy that matches the given identity, facet, and this object adapter.
            """
            raise NotImplementedError("method 'addFacet' not implemented")

        def addWithUUID(self, servant):
            """
            Add a servant to this object adapter's Active Servant Map,
            using an automatically generated UUID as its identity. Note that
            the generated UUID identity can be accessed using the proxy's
            ice_getIdentity operation.
            Arguments:
            servant -- The servant to add.
            Returns: A proxy that matches the generated UUID identity and this object adapter.
            """
            raise NotImplementedError("method 'addWithUUID' not implemented")

        def addFacetWithUUID(self, servant, facet):
            """
            Like addWithUUID, but with a facet. Calling
            addWithUUID(servant) is equivalent to calling
            addFacetWithUUID with an empty facet.
            Arguments:
            servant -- The servant to add.
            facet -- The facet. An empty facet means the default facet.
            Returns: A proxy that matches the generated UUID identity, facet, and this object adapter.
            """
            raise NotImplementedError("method 'addFacetWithUUID' not implemented")

        def addDefaultServant(self, servant, category):
            """
            Add a default servant to handle requests for a specific
            category. Adding a default servant for a category for
            which a default servant is already registered throws
            AlreadyRegisteredException. To dispatch operation
            calls on servants, the object adapter tries to find a servant
            for a given Ice object identity and facet in the following
            order:
            The object adapter tries to find a servant for the identity
            and facet in the Active Servant Map.
            If no servant has been found in the Active Servant Map, the
            object adapter tries to find a default servant for the category
            component of the identity.
            If no servant has been found by any of the preceding steps,
            the object adapter tries to find a default servant for an empty
            category, regardless of the category contained in the identity.
            If no servant has been found by any of the preceding steps,
            the object adapter gives up and the caller receives
            ObjectNotExistException or FacetNotExistException.
            Arguments:
            servant -- The default servant.
            category -- The category for which the default servant is registered. An empty category means it will handle all categories.
            """
            raise NotImplementedError("method 'addDefaultServant' not implemented")

        def remove(self, id):
            """
            Remove a servant (that is, the default facet) from the object
            adapter's Active Servant Map.
            Arguments:
            id -- The identity of the Ice object that is implemented by the servant. If the servant implements multiple Ice objects, remove has to be called for all those Ice objects. Removing an identity that is not in the map throws NotRegisteredException.
            Returns: The removed servant.
            """
            raise NotImplementedError("method 'remove' not implemented")

        def removeFacet(self, id, facet):
            """
            Like remove, but with a facet. Calling remove(id)
            is equivalent to calling removeFacet with an empty facet.
            Arguments:
            id -- The identity of the Ice object that is implemented by the servant.
            facet -- The facet. An empty facet means the default facet.
            Returns: The removed servant.
            """
            raise NotImplementedError("method 'removeFacet' not implemented")

        def removeAllFacets(self, id):
            """
            Remove all facets with the given identity from the Active
            Servant Map. The operation completely removes the Ice object,
            including its default facet. Removing an identity that
            is not in the map throws NotRegisteredException.
            Arguments:
            id -- The identity of the Ice object to be removed.
            Returns: A collection containing all the facet names and servants of the removed Ice object.
            """
            raise NotImplementedError("method 'removeAllFacets' not implemented")

        def removeDefaultServant(self, category):
            """
            Remove the default servant for a specific category. Attempting
            to remove a default servant for a category that is not
            registered throws NotRegisteredException.
            Arguments:
            category -- The category of the default servant to remove.
            Returns: The default servant.
            """
            raise NotImplementedError("method 'removeDefaultServant' not implemented")

        def find(self, id):
            """
            Look up a servant in this object adapter's Active Servant Map
            by the identity of the Ice object it implements.
            This operation only tries to look up a servant in
            the Active Servant Map. It does not attempt to find a servant
            by using any installed ServantLocator.
            Arguments:
            id -- The identity of the Ice object for which the servant should be returned.
            Returns: The servant that implements the Ice object with the given identity, or null if no such servant has been found.
            """
            raise NotImplementedError("method 'find' not implemented")

        def findFacet(self, id, facet):
            """
            Like find, but with a facet. Calling find(id)
            is equivalent to calling findFacet with an empty
            facet.
            Arguments:
            id -- The identity of the Ice object for which the servant should be returned.
            facet -- The facet. An empty facet means the default facet.
            Returns: The servant that implements the Ice object with the given identity and facet, or null if no such servant has been found.
            """
            raise NotImplementedError("method 'findFacet' not implemented")

        def findAllFacets(self, id):
            """
            Find all facets with the given identity in the Active Servant
            Map.
            Arguments:
            id -- The identity of the Ice object for which the facets should be returned.
            Returns: A collection containing all the facet names and servants that have been found, or an empty map if there is no facet for the given identity.
            """
            raise NotImplementedError("method 'findAllFacets' not implemented")

        def findByProxy(self, proxy):
            """
            Look up a servant in this object adapter's Active Servant Map,
            given a proxy.
            This operation only tries to lookup a servant in
            the Active Servant Map. It does not attempt to find a servant
            by using any installed ServantLocator.
            Arguments:
            proxy -- The proxy for which the servant should be returned.
            Returns: The servant that matches the proxy, or null if no such servant has been found.
            """
            raise NotImplementedError("method 'findByProxy' not implemented")

        def addServantLocator(self, locator, category):
            """
            Add a Servant Locator to this object adapter. Adding a servant
            locator for a category for which a servant locator is already
            registered throws AlreadyRegisteredException. To dispatch
            operation calls on servants, the object adapter tries to find a
            servant for a given Ice object identity and facet in the
            following order:
            The object adapter tries to find a servant for the identity
            and facet in the Active Servant Map.
            If no servant has been found in the Active Servant Map,
            the object adapter tries to find a servant locator for the
            category component of the identity. If a locator is found, the
            object adapter tries to find a servant using this locator.
            If no servant has been found by any of the preceding steps,
            the object adapter tries to find a locator for an empty category,
            regardless of the category contained in the identity. If a
            locator is found, the object adapter tries to find a servant
            using this locator.
            If no servant has been found by any of the preceding steps,
            the object adapter gives up and the caller receives
            ObjectNotExistException or FacetNotExistException.
            Only one locator for the empty category can be
            installed.
            Arguments:
            locator -- The locator to add.
            category -- The category for which the Servant Locator can locate servants, or an empty string if the Servant Locator does not belong to any specific category.
            """
            raise NotImplementedError("method 'addServantLocator' not implemented")

        def removeServantLocator(self, category):
            """
            Remove a Servant Locator from this object adapter.
            Arguments:
            category -- The category for which the Servant Locator can locate servants, or an empty string if the Servant Locator does not belong to any specific category.
            Returns: The Servant Locator, or throws NotRegisteredException if no Servant Locator was found for the given category.
            """
            raise NotImplementedError("method 'removeServantLocator' not implemented")

        def findServantLocator(self, category):
            """
            Find a Servant Locator installed with this object adapter.
            Arguments:
            category -- The category for which the Servant Locator can locate servants, or an empty string if the Servant Locator does not belong to any specific category.
            Returns: The Servant Locator, or null if no Servant Locator was found for the given category.
            """
            raise NotImplementedError("method 'findServantLocator' not implemented")

        def findDefaultServant(self, category):
            """
            Find the default servant for a specific category.
            Arguments:
            category -- The category of the default servant to find.
            Returns: The default servant or null if no default servant was registered for the category.
            """
            raise NotImplementedError("method 'findDefaultServant' not implemented")

        def createProxy(self, id):
            """
            Create a proxy for the object with the given identity. If this
            object adapter is configured with an adapter id, the return
            value is an indirect proxy that refers to the adapter id. If
            a replica group id is also defined, the return value is an
            indirect proxy that refers to the replica group id. Otherwise,
            if no adapter id is defined, the return value is a direct
            proxy containing this object adapter's published endpoints.
            Arguments:
            id -- The object's identity.
            Returns: A proxy for the object with the given identity.
            """
            raise NotImplementedError("method 'createProxy' not implemented")

        def createDirectProxy(self, id):
            """
            Create a direct proxy for the object with the given identity.
            The returned proxy contains this object adapter's published
            endpoints.
            Arguments:
            id -- The object's identity.
            Returns: A proxy for the object with the given identity.
            """
            raise NotImplementedError("method 'createDirectProxy' not implemented")

        def createIndirectProxy(self, id):
            """
            Create an indirect proxy for the object with the given identity.
            If this object adapter is configured with an adapter id, the
            return value refers to the adapter id. Otherwise, the return
            value contains only the object identity.
            Arguments:
            id -- The object's identity.
            Returns: A proxy for the object with the given identity.
            """
            raise NotImplementedError("method 'createIndirectProxy' not implemented")

        def setLocator(self, loc):
            """
            Set an Ice locator for this object adapter. By doing so, the
            object adapter will register itself with the locator registry
            when it is activated for the first time. Furthermore, the proxies
            created by this object adapter will contain the adapter identifier
            instead of its endpoints. The adapter identifier must be configured
            using the AdapterId property.
            Arguments:
            loc -- The locator used by this object adapter.
            """
            raise NotImplementedError("method 'setLocator' not implemented")

        def getLocator(self):
            """
            Get the Ice locator used by this object adapter.
            Returns: The locator used by this object adapter, or null if no locator is used by this object adapter.
            """
            raise NotImplementedError("method 'getLocator' not implemented")

        def getEndpoints(self):
            """
            Get the set of endpoints configured with this object adapter.
            Returns: The set of endpoints.
            """
            raise NotImplementedError("method 'getEndpoints' not implemented")

        def refreshPublishedEndpoints(self):
            """
            Refresh the set of published endpoints. The run time re-reads
            the PublishedEndpoints property if it is set and re-reads the
            list of local interfaces if the adapter is configured to listen
            on all endpoints. This operation is useful to refresh the endpoint
            information that is published in the proxies that are created by
            an object adapter if the network interfaces used by a host changes.
            """
            raise NotImplementedError("method 'refreshPublishedEndpoints' not implemented")

        def getPublishedEndpoints(self):
            """
            Get the set of endpoints that proxies created by this object
            adapter will contain.
            Returns: The set of published endpoints.
            """
            raise NotImplementedError("method 'getPublishedEndpoints' not implemented")

        def setPublishedEndpoints(self, newEndpoints):
            """
            Set of the endpoints that proxies created by this object
            adapter will contain.
            Arguments:
            newEndpoints -- The new set of endpoints that the object adapter will embed in proxies.
            """
            raise NotImplementedError("method 'setPublishedEndpoints' not implemented")

        def __str__(self):
            return IcePy.stringify(self, _M_Ice._t_ObjectAdapter)

        __repr__ = __str__

    _M_Ice._t_ObjectAdapter = IcePy.defineValue('::Ice::ObjectAdapter', ObjectAdapter, -1, (), False, True, None, ())
    ObjectAdapter._ice_type = _M_Ice._t_ObjectAdapter

    _M_Ice.ObjectAdapter = ObjectAdapter
    del ObjectAdapter

# End of module Ice



# -*- coding: utf-8 -*-
#
# Copyright (c) ZeroC, Inc. All rights reserved.
#
#
# Ice version 3.7.6
#
# <auto-generated>
#
# Generated from file `Current.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import Ice.ObjectAdapterF_ice
import Ice.ConnectionF_ice
import Ice.Identity_ice
import Ice.Version_ice

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Start of module Ice
__name__ = 'Ice'

if '_t_Context' not in _M_Ice.__dict__:
    _M_Ice._t_Context = IcePy.defineDictionary('::Ice::Context', (), IcePy._t_string, IcePy._t_string)

if 'OperationMode' not in _M_Ice.__dict__:
    _M_Ice.OperationMode = Ice.createTempClass()
    class OperationMode(Ice.EnumBase):
        """
        Determines the retry behavior an invocation in case of a (potentially) recoverable error.
        Enumerators:
        Normal -- Ordinary operations have Normal mode.  These operations
        modify object state; invoking such an operation twice in a row
        has different semantics than invoking it once. The Ice run time
        guarantees that it will not violate at-most-once semantics for
        Normal operations.
        Nonmutating -- Operations that use the Slice nonmutating keyword must not
        modify object state. For C++, nonmutating operations generate
        const member functions in the skeleton. In addition, the Ice
        run time will attempt to transparently recover from certain
        run-time errors by re-issuing a failed request and propagate
        the failure to the application only if the second attempt
        fails.
        Nonmutating is deprecated; Use the
        idempotent keyword instead. For C++, to retain the mapping
        of nonmutating operations to C++ const
        member functions, use the \["cpp:const"] metadata
        directive.
        Idempotent -- Operations that use the Slice idempotent keyword can modify
        object state, but invoking an operation twice in a row must
        result in the same object state as invoking it once.  For
        example, x = 1 is an idempotent statement,
        whereas x += 1 is not. For idempotent
        operations, the Ice run-time uses the same retry behavior
        as for nonmutating operations in case of a potentially
        recoverable error.
        """

        def __init__(self, _n, _v):
            Ice.EnumBase.__init__(self, _n, _v)

        def valueOf(self, _n):
            if _n in self._enumerators:
                return self._enumerators[_n]
            return None
        valueOf = classmethod(valueOf)

    OperationMode.Normal = OperationMode("Normal", 0)
    OperationMode.Nonmutating = OperationMode("Nonmutating", 1)
    OperationMode.Idempotent = OperationMode("Idempotent", 2)
    OperationMode._enumerators = { 0:OperationMode.Normal, 1:OperationMode.Nonmutating, 2:OperationMode.Idempotent }

    _M_Ice._t_OperationMode = IcePy.defineEnum('::Ice::OperationMode', OperationMode, (), OperationMode._enumerators)

    _M_Ice.OperationMode = OperationMode
    del OperationMode

if 'Current' not in _M_Ice.__dict__:
    _M_Ice.Current = Ice.createTempClass()
    class Current(object):
        """
        Information about the current method invocation for servers. Each
        operation on the server has a Current as its implicit final
        parameter. Current is mostly used for Ice services. Most
        applications ignore this parameter.
        Members:
        adapter -- The object adapter.
        con -- Information about the connection over which the current method
        invocation was received. If the invocation is direct due to
        collocation optimization, this value is set to null.
        id -- The Ice object identity.
        facet -- The facet.
        operation -- The operation name.
        mode -- The mode of the operation.
        ctx -- The request context, as received from the client.
        requestId -- The request id unless oneway (0).
        encoding -- The encoding version used to encode the input and output parameters.
        """
        def __init__(self, adapter=None, con=None, id=Ice._struct_marker, facet='', operation='', mode=_M_Ice.OperationMode.Normal, ctx=None, requestId=0, encoding=Ice._struct_marker):
            self.adapter = adapter
            self.con = con
            if id is Ice._struct_marker:
                self.id = _M_Ice.Identity()
            else:
                self.id = id
            self.facet = facet
            self.operation = operation
            self.mode = mode
            self.ctx = ctx
            self.requestId = requestId
            if encoding is Ice._struct_marker:
                self.encoding = _M_Ice.EncodingVersion()
            else:
                self.encoding = encoding

        def __eq__(self, other):
            if other is None:
                return False
            elif not isinstance(other, _M_Ice.Current):
                return NotImplemented
            else:
                if self.adapter != other.adapter:
                    return False
                if self.con != other.con:
                    return False
                if self.id != other.id:
                    return False
                if self.facet != other.facet:
                    return False
                if self.operation != other.operation:
                    return False
                if self.mode != other.mode:
                    return False
                if self.ctx != other.ctx:
                    return False
                if self.requestId != other.requestId:
                    return False
                if self.encoding != other.encoding:
                    return False
                return True

        def __ne__(self, other):
            return not self.__eq__(other)

        def __str__(self):
            return IcePy.stringify(self, _M_Ice._t_Current)

        __repr__ = __str__

    _M_Ice._t_Current = IcePy.defineStruct('::Ice::Current', Current, (), (
        ('adapter', (), _M_Ice._t_ObjectAdapter),
        ('con', (), _M_Ice._t_Connection),
        ('id', (), _M_Ice._t_Identity),
        ('facet', (), IcePy._t_string),
        ('operation', (), IcePy._t_string),
        ('mode', (), _M_Ice._t_OperationMode),
        ('ctx', (), _M_Ice._t_Context),
        ('requestId', (), IcePy._t_int),
        ('encoding', (), _M_Ice._t_EncodingVersion)
    ))

    _M_Ice.Current = Current
    del Current

# End of module Ice

Ice.sliceChecksums["::Ice::Context"] = "e6cb8aba8a3ca160eab3597c6fbbeba"
Ice.sliceChecksums["::Ice::OperationMode"] = "56db1e0dd464f97828282bdb11d8955"



# -*- coding: utf-8 -*-
#
# Copyright (c) ZeroC, Inc. All rights reserved.
#
#
# Ice version 3.7.6
#
# <auto-generated>
#
# Generated from file `RemoteLogger.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import Ice.BuiltinSequences_ice

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Start of module Ice
__name__ = 'Ice'

if 'LogMessageType' not in _M_Ice.__dict__:
    _M_Ice.LogMessageType = Ice.createTempClass()
    class LogMessageType(Ice.EnumBase):
        """
        An enumeration representing the different types of log messages.
        Enumerators:
        PrintMessage -- The Logger received a print message.
        TraceMessage -- The Logger received a trace message.
        WarningMessage -- The Logger received a warning message.
        ErrorMessage -- The Logger received an error message.
        """

        def __init__(self, _n, _v):
            Ice.EnumBase.__init__(self, _n, _v)

        def valueOf(self, _n):
            if _n in self._enumerators:
                return self._enumerators[_n]
            return None
        valueOf = classmethod(valueOf)

    LogMessageType.PrintMessage = LogMessageType("PrintMessage", 0)
    LogMessageType.TraceMessage = LogMessageType("TraceMessage", 1)
    LogMessageType.WarningMessage = LogMessageType("WarningMessage", 2)
    LogMessageType.ErrorMessage = LogMessageType("ErrorMessage", 3)
    LogMessageType._enumerators = { 0:LogMessageType.PrintMessage, 1:LogMessageType.TraceMessage, 2:LogMessageType.WarningMessage, 3:LogMessageType.ErrorMessage }

    _M_Ice._t_LogMessageType = IcePy.defineEnum('::Ice::LogMessageType', LogMessageType, (), LogMessageType._enumerators)

    _M_Ice.LogMessageType = LogMessageType
    del LogMessageType

if '_t_LogMessageTypeSeq' not in _M_Ice.__dict__:
    _M_Ice._t_LogMessageTypeSeq = IcePy.defineSequence('::Ice::LogMessageTypeSeq', (), _M_Ice._t_LogMessageType)

if 'LogMessage' not in _M_Ice.__dict__:
    _M_Ice.LogMessage = Ice.createTempClass()
    class LogMessage(object):
        """
        A complete log message.
        Members:
        type -- The type of message sent to the Logger.
        timestamp -- The date and time when the Logger received this message, expressed
        as the number of microseconds since the Unix Epoch (00:00:00 UTC on 1 January 1970)
        traceCategory -- For a message of type trace, the trace category of this log message;
        otherwise, the empty string.
        message -- The log message itself.
        """
        def __init__(self, type=_M_Ice.LogMessageType.PrintMessage, timestamp=0, traceCategory='', message=''):
            self.type = type
            self.timestamp = timestamp
            self.traceCategory = traceCategory
            self.message = message

        def __hash__(self):
            _h = 0
            _h = 5 * _h + Ice.getHash(self.type)
            _h = 5 * _h + Ice.getHash(self.timestamp)
            _h = 5 * _h + Ice.getHash(self.traceCategory)
            _h = 5 * _h + Ice.getHash(self.message)
            return _h % 0x7fffffff

        def __compare(self, other):
            if other is None:
                return 1
            elif not isinstance(other, _M_Ice.LogMessage):
                return NotImplemented
            else:
                if self.type is None or other.type is None:
                    if self.type != other.type:
                        return (-1 if self.type is None else 1)
                else:
                    if self.type < other.type:
                        return -1
                    elif self.type > other.type:
                        return 1
                if self.timestamp is None or other.timestamp is None:
                    if self.timestamp != other.timestamp:
                        return (-1 if self.timestamp is None else 1)
                else:
                    if self.timestamp < other.timestamp:
                        return -1
                    elif self.timestamp > other.timestamp:
                        return 1
                if self.traceCategory is None or other.traceCategory is None:
                    if self.traceCategory != other.traceCategory:
                        return (-1 if self.traceCategory is None else 1)
                else:
                    if self.traceCategory < other.traceCategory:
                        return -1
                    elif self.traceCategory > other.traceCategory:
                        return 1
                if self.message is None or other.message is None:
                    if self.message != other.message:
                        return (-1 if self.message is None else 1)
                else:
                    if self.message < other.message:
                        return -1
                    elif self.message > other.message:
                        return 1
                return 0

        def __lt__(self, other):
            r = self.__compare(other)
            if r is NotImplemented:
                return r
            else:
                return r < 0

        def __le__(self, other):
            r = self.__compare(other)
            if r is NotImplemented:
                return r
            else:
                return r <= 0

        def __gt__(self, other):
            r = self.__compare(other)
            if r is NotImplemented:
                return r
            else:
                return r > 0

        def __ge__(self, other):
            r = self.__compare(other)
            if r is NotImplemented:
                return r
            else:
                return r >= 0

        def __eq__(self, other):
            r = self.__compare(other)
            if r is NotImplemented:
                return r
            else:
                return r == 0

        def __ne__(self, other):
            r = self.__compare(other)
            if r is NotImplemented:
                return r
            else:
                return r != 0

        def __str__(self):
            return IcePy.stringify(self, _M_Ice._t_LogMessage)

        __repr__ = __str__

    _M_Ice._t_LogMessage = IcePy.defineStruct('::Ice::LogMessage', LogMessage, (), (
        ('type', (), _M_Ice._t_LogMessageType),
        ('timestamp', (), IcePy._t_long),
        ('traceCategory', (), IcePy._t_string),
        ('message', (), IcePy._t_string)
    ))

    _M_Ice.LogMessage = LogMessage
    del LogMessage

if '_t_LogMessageSeq' not in _M_Ice.__dict__:
    _M_Ice._t_LogMessageSeq = IcePy.defineSequence('::Ice::LogMessageSeq', (), _M_Ice._t_LogMessage)

_M_Ice._t_RemoteLogger = IcePy.defineValue('::Ice::RemoteLogger', Ice.Value, -1, (), False, True, None, ())

if 'RemoteLoggerPrx' not in _M_Ice.__dict__:
    _M_Ice.RemoteLoggerPrx = Ice.createTempClass()
    class RemoteLoggerPrx(Ice.ObjectPrx):

        """
        init is called by attachRemoteLogger when a RemoteLogger proxy is attached.
        Arguments:
        prefix -- The prefix of the associated local Logger.
        logMessages -- Old log messages generated before "now".
        context -- The request context for the invocation.
        """
        def init(self, prefix, logMessages, context=None):
            return _M_Ice.RemoteLogger._op_init.invoke(self, ((prefix, logMessages), context))

        """
        init is called by attachRemoteLogger when a RemoteLogger proxy is attached.
        Arguments:
        prefix -- The prefix of the associated local Logger.
        logMessages -- Old log messages generated before "now".
        context -- The request context for the invocation.
        Returns: A future object for the invocation.
        """
        def initAsync(self, prefix, logMessages, context=None):
            return _M_Ice.RemoteLogger._op_init.invokeAsync(self, ((prefix, logMessages), context))

        """
        init is called by attachRemoteLogger when a RemoteLogger proxy is attached.
        Arguments:
        prefix -- The prefix of the associated local Logger.
        logMessages -- Old log messages generated before "now".
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        context -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_init(self, prefix, logMessages, _response=None, _ex=None, _sent=None, context=None):
            return _M_Ice.RemoteLogger._op_init.begin(self, ((prefix, logMessages), _response, _ex, _sent, context))

        """
        init is called by attachRemoteLogger when a RemoteLogger proxy is attached.
        Arguments:
        prefix -- The prefix of the associated local Logger.
        logMessages -- Old log messages generated before "now".
        """
        def end_init(self, _r):
            return _M_Ice.RemoteLogger._op_init.end(self, _r)

        """
        Log a LogMessage. Note that log may be called by LoggerAdmin before init.
        Arguments:
        message -- The message to log.
        context -- The request context for the invocation.
        """
        def log(self, message, context=None):
            return _M_Ice.RemoteLogger._op_log.invoke(self, ((message, ), context))

        """
        Log a LogMessage. Note that log may be called by LoggerAdmin before init.
        Arguments:
        message -- The message to log.
        context -- The request context for the invocation.
        Returns: A future object for the invocation.
        """
        def logAsync(self, message, context=None):
            return _M_Ice.RemoteLogger._op_log.invokeAsync(self, ((message, ), context))

        """
        Log a LogMessage. Note that log may be called by LoggerAdmin before init.
        Arguments:
        message -- The message to log.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        context -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_log(self, message, _response=None, _ex=None, _sent=None, context=None):
            return _M_Ice.RemoteLogger._op_log.begin(self, ((message, ), _response, _ex, _sent, context))

        """
        Log a LogMessage. Note that log may be called by LoggerAdmin before init.
        Arguments:
        message -- The message to log.
        """
        def end_log(self, _r):
            return _M_Ice.RemoteLogger._op_log.end(self, _r)

        @staticmethod
        def checkedCast(proxy, facetOrContext=None, context=None):
            return _M_Ice.RemoteLoggerPrx.ice_checkedCast(proxy, '::Ice::RemoteLogger', facetOrContext, context)

        @staticmethod
        def uncheckedCast(proxy, facet=None):
            return _M_Ice.RemoteLoggerPrx.ice_uncheckedCast(proxy, facet)

        @staticmethod
        def ice_staticId():
            return '::Ice::RemoteLogger'
    _M_Ice._t_RemoteLoggerPrx = IcePy.defineProxy('::Ice::RemoteLogger', RemoteLoggerPrx)

    _M_Ice.RemoteLoggerPrx = RemoteLoggerPrx
    del RemoteLoggerPrx

    _M_Ice.RemoteLogger = Ice.createTempClass()
    class RemoteLogger(Ice.Object):

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::Ice::RemoteLogger')

        def ice_id(self, current=None):
            return '::Ice::RemoteLogger'

        @staticmethod
        def ice_staticId():
            return '::Ice::RemoteLogger'

        def init(self, prefix, logMessages, current=None):
            """
            init is called by attachRemoteLogger when a RemoteLogger proxy is attached.
            Arguments:
            prefix -- The prefix of the associated local Logger.
            logMessages -- Old log messages generated before "now".
            current -- The Current object for the invocation.
            Returns: A future object for the invocation.
            """
            raise NotImplementedError("servant method 'init' not implemented")

        def log(self, message, current=None):
            """
            Log a LogMessage. Note that log may be called by LoggerAdmin before init.
            Arguments:
            message -- The message to log.
            current -- The Current object for the invocation.
            Returns: A future object for the invocation.
            """
            raise NotImplementedError("servant method 'log' not implemented")

        def __str__(self):
            return IcePy.stringify(self, _M_Ice._t_RemoteLoggerDisp)

        __repr__ = __str__

    _M_Ice._t_RemoteLoggerDisp = IcePy.defineClass('::Ice::RemoteLogger', RemoteLogger, (), None, ())
    RemoteLogger._ice_type = _M_Ice._t_RemoteLoggerDisp

    RemoteLogger._op_init = IcePy.Operation('init', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_string, False, 0), ((), _M_Ice._t_LogMessageSeq, False, 0)), (), None, ())
    RemoteLogger._op_log = IcePy.Operation('log', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_Ice._t_LogMessage, False, 0),), (), None, ())

    _M_Ice.RemoteLogger = RemoteLogger
    del RemoteLogger

if 'RemoteLoggerAlreadyAttachedException' not in _M_Ice.__dict__:
    _M_Ice.RemoteLoggerAlreadyAttachedException = Ice.createTempClass()
    class RemoteLoggerAlreadyAttachedException(Ice.UserException):
        """
        Thrown when the provided RemoteLogger was previously attached to a LoggerAdmin.
        """
        def __init__(self):
            pass

        def __str__(self):
            return IcePy.stringifyException(self)

        __repr__ = __str__

        _ice_id = '::Ice::RemoteLoggerAlreadyAttachedException'

    _M_Ice._t_RemoteLoggerAlreadyAttachedException = IcePy.defineException('::Ice::RemoteLoggerAlreadyAttachedException', RemoteLoggerAlreadyAttachedException, (), False, None, ())
    RemoteLoggerAlreadyAttachedException._ice_type = _M_Ice._t_RemoteLoggerAlreadyAttachedException

    _M_Ice.RemoteLoggerAlreadyAttachedException = RemoteLoggerAlreadyAttachedException
    del RemoteLoggerAlreadyAttachedException

_M_Ice._t_LoggerAdmin = IcePy.defineValue('::Ice::LoggerAdmin', Ice.Value, -1, (), False, True, None, ())

if 'LoggerAdminPrx' not in _M_Ice.__dict__:
    _M_Ice.LoggerAdminPrx = Ice.createTempClass()
    class LoggerAdminPrx(Ice.ObjectPrx):

        """
        Attaches a RemoteLogger object to the local logger.
        attachRemoteLogger calls init on the provided RemoteLogger proxy.
        Arguments:
        prx -- A proxy to the remote logger.
        messageTypes -- The list of message types that the remote logger wishes to receive. An empty list means no filtering (send all message types).
        traceCategories -- The categories of traces that the remote logger wishes to receive. This parameter is ignored if messageTypes is not empty and does not include trace. An empty list means no filtering (send all trace categories).
        messageMax -- The maximum number of log messages (of all types) to be provided to init. A negative value requests all messages available.
        context -- The request context for the invocation.
        Throws:
        RemoteLoggerAlreadyAttachedException -- Raised if this remote logger is already attached to this admin object.
        """
        def attachRemoteLogger(self, prx, messageTypes, traceCategories, messageMax, context=None):
            return _M_Ice.LoggerAdmin._op_attachRemoteLogger.invoke(self, ((prx, messageTypes, traceCategories, messageMax), context))

        """
        Attaches a RemoteLogger object to the local logger.
        attachRemoteLogger calls init on the provided RemoteLogger proxy.
        Arguments:
        prx -- A proxy to the remote logger.
        messageTypes -- The list of message types that the remote logger wishes to receive. An empty list means no filtering (send all message types).
        traceCategories -- The categories of traces that the remote logger wishes to receive. This parameter is ignored if messageTypes is not empty and does not include trace. An empty list means no filtering (send all trace categories).
        messageMax -- The maximum number of log messages (of all types) to be provided to init. A negative value requests all messages available.
        context -- The request context for the invocation.
        Returns: A future object for the invocation.
        """
        def attachRemoteLoggerAsync(self, prx, messageTypes, traceCategories, messageMax, context=None):
            return _M_Ice.LoggerAdmin._op_attachRemoteLogger.invokeAsync(self, ((prx, messageTypes, traceCategories, messageMax), context))

        """
        Attaches a RemoteLogger object to the local logger.
        attachRemoteLogger calls init on the provided RemoteLogger proxy.
        Arguments:
        prx -- A proxy to the remote logger.
        messageTypes -- The list of message types that the remote logger wishes to receive. An empty list means no filtering (send all message types).
        traceCategories -- The categories of traces that the remote logger wishes to receive. This parameter is ignored if messageTypes is not empty and does not include trace. An empty list means no filtering (send all trace categories).
        messageMax -- The maximum number of log messages (of all types) to be provided to init. A negative value requests all messages available.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        context -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_attachRemoteLogger(self, prx, messageTypes, traceCategories, messageMax, _response=None, _ex=None, _sent=None, context=None):
            return _M_Ice.LoggerAdmin._op_attachRemoteLogger.begin(self, ((prx, messageTypes, traceCategories, messageMax), _response, _ex, _sent, context))

        """
        Attaches a RemoteLogger object to the local logger.
        attachRemoteLogger calls init on the provided RemoteLogger proxy.
        Arguments:
        prx -- A proxy to the remote logger.
        messageTypes -- The list of message types that the remote logger wishes to receive. An empty list means no filtering (send all message types).
        traceCategories -- The categories of traces that the remote logger wishes to receive. This parameter is ignored if messageTypes is not empty and does not include trace. An empty list means no filtering (send all trace categories).
        messageMax -- The maximum number of log messages (of all types) to be provided to init. A negative value requests all messages available.
        Throws:
        RemoteLoggerAlreadyAttachedException -- Raised if this remote logger is already attached to this admin object.
        """
        def end_attachRemoteLogger(self, _r):
            return _M_Ice.LoggerAdmin._op_attachRemoteLogger.end(self, _r)

        """
        Detaches a RemoteLogger object from the local logger.
        Arguments:
        prx -- A proxy to the remote logger.
        context -- The request context for the invocation.
        Returns: True if the provided remote logger proxy was detached, and false otherwise.
        """
        def detachRemoteLogger(self, prx, context=None):
            return _M_Ice.LoggerAdmin._op_detachRemoteLogger.invoke(self, ((prx, ), context))

        """
        Detaches a RemoteLogger object from the local logger.
        Arguments:
        prx -- A proxy to the remote logger.
        context -- The request context for the invocation.
        Returns: A future object for the invocation.
        """
        def detachRemoteLoggerAsync(self, prx, context=None):
            return _M_Ice.LoggerAdmin._op_detachRemoteLogger.invokeAsync(self, ((prx, ), context))

        """
        Detaches a RemoteLogger object from the local logger.
        Arguments:
        prx -- A proxy to the remote logger.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        context -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_detachRemoteLogger(self, prx, _response=None, _ex=None, _sent=None, context=None):
            return _M_Ice.LoggerAdmin._op_detachRemoteLogger.begin(self, ((prx, ), _response, _ex, _sent, context))

        """
        Detaches a RemoteLogger object from the local logger.
        Arguments:
        prx -- A proxy to the remote logger.
        Returns: True if the provided remote logger proxy was detached, and false otherwise.
        """
        def end_detachRemoteLogger(self, _r):
            return _M_Ice.LoggerAdmin._op_detachRemoteLogger.end(self, _r)

        """
        Retrieves log messages recently logged.
        Arguments:
        messageTypes -- The list of message types that the caller wishes to receive. An empty list means no filtering (send all message types).
        traceCategories -- The categories of traces that caller wish to receive. This parameter is ignored if messageTypes is not empty and does not include trace. An empty list means no filtering (send all trace categories).
        messageMax -- The maximum number of log messages (of all types) to be returned. A negative value requests all messages available.
        context -- The request context for the invocation.
        Returns a tuple containing the following:
        _retval -- The Log messages.
        prefix -- The prefix of the associated local logger.
        """
        def getLog(self, messageTypes, traceCategories, messageMax, context=None):
            return _M_Ice.LoggerAdmin._op_getLog.invoke(self, ((messageTypes, traceCategories, messageMax), context))

        """
        Retrieves log messages recently logged.
        Arguments:
        messageTypes -- The list of message types that the caller wishes to receive. An empty list means no filtering (send all message types).
        traceCategories -- The categories of traces that caller wish to receive. This parameter is ignored if messageTypes is not empty and does not include trace. An empty list means no filtering (send all trace categories).
        messageMax -- The maximum number of log messages (of all types) to be returned. A negative value requests all messages available.
        context -- The request context for the invocation.
        Returns: A future object for the invocation.
        """
        def getLogAsync(self, messageTypes, traceCategories, messageMax, context=None):
            return _M_Ice.LoggerAdmin._op_getLog.invokeAsync(self, ((messageTypes, traceCategories, messageMax), context))

        """
        Retrieves log messages recently logged.
        Arguments:
        messageTypes -- The list of message types that the caller wishes to receive. An empty list means no filtering (send all message types).
        traceCategories -- The categories of traces that caller wish to receive. This parameter is ignored if messageTypes is not empty and does not include trace. An empty list means no filtering (send all trace categories).
        messageMax -- The maximum number of log messages (of all types) to be returned. A negative value requests all messages available.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        context -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getLog(self, messageTypes, traceCategories, messageMax, _response=None, _ex=None, _sent=None, context=None):
            return _M_Ice.LoggerAdmin._op_getLog.begin(self, ((messageTypes, traceCategories, messageMax), _response, _ex, _sent, context))

        """
        Retrieves log messages recently logged.
        Arguments:
        messageTypes -- The list of message types that the caller wishes to receive. An empty list means no filtering (send all message types).
        traceCategories -- The categories of traces that caller wish to receive. This parameter is ignored if messageTypes is not empty and does not include trace. An empty list means no filtering (send all trace categories).
        messageMax -- The maximum number of log messages (of all types) to be returned. A negative value requests all messages available.
        Returns a tuple containing the following:
        _retval -- The Log messages.
        prefix -- The prefix of the associated local logger.
        """
        def end_getLog(self, _r):
            return _M_Ice.LoggerAdmin._op_getLog.end(self, _r)

        @staticmethod
        def checkedCast(proxy, facetOrContext=None, context=None):
            return _M_Ice.LoggerAdminPrx.ice_checkedCast(proxy, '::Ice::LoggerAdmin', facetOrContext, context)

        @staticmethod
        def uncheckedCast(proxy, facet=None):
            return _M_Ice.LoggerAdminPrx.ice_uncheckedCast(proxy, facet)

        @staticmethod
        def ice_staticId():
            return '::Ice::LoggerAdmin'
    _M_Ice._t_LoggerAdminPrx = IcePy.defineProxy('::Ice::LoggerAdmin', LoggerAdminPrx)

    _M_Ice.LoggerAdminPrx = LoggerAdminPrx
    del LoggerAdminPrx

    _M_Ice.LoggerAdmin = Ice.createTempClass()
    class LoggerAdmin(Ice.Object):

        def ice_ids(self, current=None):
            return ('::Ice::LoggerAdmin', '::Ice::Object')

        def ice_id(self, current=None):
            return '::Ice::LoggerAdmin'

        @staticmethod
        def ice_staticId():
            return '::Ice::LoggerAdmin'

        def attachRemoteLogger(self, prx, messageTypes, traceCategories, messageMax, current=None):
            """
            Attaches a RemoteLogger object to the local logger.
            attachRemoteLogger calls init on the provided RemoteLogger proxy.
            Arguments:
            prx -- A proxy to the remote logger.
            messageTypes -- The list of message types that the remote logger wishes to receive. An empty list means no filtering (send all message types).
            traceCategories -- The categories of traces that the remote logger wishes to receive. This parameter is ignored if messageTypes is not empty and does not include trace. An empty list means no filtering (send all trace categories).
            messageMax -- The maximum number of log messages (of all types) to be provided to init. A negative value requests all messages available.
            current -- The Current object for the invocation.
            Returns: A future object for the invocation.
            Throws:
            RemoteLoggerAlreadyAttachedException -- Raised if this remote logger is already attached to this admin object.
            """
            raise NotImplementedError("servant method 'attachRemoteLogger' not implemented")

        def detachRemoteLogger(self, prx, current=None):
            """
            Detaches a RemoteLogger object from the local logger.
            Arguments:
            prx -- A proxy to the remote logger.
            current -- The Current object for the invocation.
            Returns: A future object for the invocation.
            """
            raise NotImplementedError("servant method 'detachRemoteLogger' not implemented")

        def getLog(self, messageTypes, traceCategories, messageMax, current=None):
            """
            Retrieves log messages recently logged.
            Arguments:
            messageTypes -- The list of message types that the caller wishes to receive. An empty list means no filtering (send all message types).
            traceCategories -- The categories of traces that caller wish to receive. This parameter is ignored if messageTypes is not empty and does not include trace. An empty list means no filtering (send all trace categories).
            messageMax -- The maximum number of log messages (of all types) to be returned. A negative value requests all messages available.
            current -- The Current object for the invocation.
            Returns: A future object for the invocation.
            """
            raise NotImplementedError("servant method 'getLog' not implemented")

        def __str__(self):
            return IcePy.stringify(self, _M_Ice._t_LoggerAdminDisp)

        __repr__ = __str__

    _M_Ice._t_LoggerAdminDisp = IcePy.defineClass('::Ice::LoggerAdmin', LoggerAdmin, (), None, ())
    LoggerAdmin._ice_type = _M_Ice._t_LoggerAdminDisp

    LoggerAdmin._op_attachRemoteLogger = IcePy.Operation('attachRemoteLogger', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_Ice._t_RemoteLoggerPrx, False, 0), ((), _M_Ice._t_LogMessageTypeSeq, False, 0), ((), _M_Ice._t_StringSeq, False, 0), ((), IcePy._t_int, False, 0)), (), None, (_M_Ice._t_RemoteLoggerAlreadyAttachedException,))
    LoggerAdmin._op_detachRemoteLogger = IcePy.Operation('detachRemoteLogger', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_Ice._t_RemoteLoggerPrx, False, 0),), (), ((), IcePy._t_bool, False, 0), ())
    LoggerAdmin._op_getLog = IcePy.Operation('getLog', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_Ice._t_LogMessageTypeSeq, False, 0), ((), _M_Ice._t_StringSeq, False, 0), ((), IcePy._t_int, False, 0)), (((), IcePy._t_string, False, 0),), ((), _M_Ice._t_LogMessageSeq, False, 0), ())

    _M_Ice.LoggerAdmin = LoggerAdmin
    del LoggerAdmin

# End of module Ice

Ice.sliceChecksums["::Ice::LogMessage"] = "b3e905f1b794f505c1f817935be760"
Ice.sliceChecksums["::Ice::LogMessageSeq"] = "26c45475ae04c87944249ec14ebe9c"
Ice.sliceChecksums["::Ice::LogMessageType"] = "938bafca5970dc253c1e82a4c8ee234b"
Ice.sliceChecksums["::Ice::LogMessageTypeSeq"] = "cb11159511adf1a74eb2901796137d7"
Ice.sliceChecksums["::Ice::LoggerAdmin"] = "cba78eeef4c84241e3677c18bbccfb0"
Ice.sliceChecksums["::Ice::RemoteLogger"] = "8c4657be5994cd2ccf54ae795b97273"
Ice.sliceChecksums["::Ice::RemoteLoggerAlreadyAttachedException"] = "c0cffbaeb5ca2d0e1d4a0f328a701b"



# -*- coding: utf-8 -*-
#
# Copyright (c) ZeroC, Inc. All rights reserved.
#
#
# Ice version 3.7.6
#
# <auto-generated>
#
# Generated from file `Metrics.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import Ice.BuiltinSequences_ice

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Start of module IceMX
_M_IceMX = Ice.openModule('IceMX')
__name__ = 'IceMX'
_M_IceMX.__doc__ = """
The Ice Management eXtension facility. It provides the
IceMX#MetricsAdmin interface for management clients to
retrieve metrics from Ice applications.
"""

if '_t_StringIntDict' not in _M_IceMX.__dict__:
    _M_IceMX._t_StringIntDict = IcePy.defineDictionary('::IceMX::StringIntDict', (), IcePy._t_string, IcePy._t_int)

if 'Metrics' not in _M_IceMX.__dict__:
    _M_IceMX.Metrics = Ice.createTempClass()
    class Metrics(Ice.Value):
        """
        The base class for metrics. A metrics object represents a
        collection of measurements associated to a given a system.
        Members:
        id -- The metrics identifier.
        total -- The total number of objects observed by this metrics. This includes
        the number of currently observed objects and the number of objects
        observed in the past.
        current -- The number of objects currently observed by this metrics.
        totalLifetime -- The sum of the lifetime of each observed objects. This does not
        include the lifetime of objects which are currently observed,
        only the objects observed in the past.
        failures -- The number of failures observed.
        """
        def __init__(self, id='', total=0, current=0, totalLifetime=0, failures=0):
            self.id = id
            self.total = total
            self.current = current
            self.totalLifetime = totalLifetime
            self.failures = failures

        def ice_id(self):
            return '::IceMX::Metrics'

        @staticmethod
        def ice_staticId():
            return '::IceMX::Metrics'

        def __str__(self):
            return IcePy.stringify(self, _M_IceMX._t_Metrics)

        __repr__ = __str__

    _M_IceMX._t_Metrics = IcePy.defineValue('::IceMX::Metrics', Metrics, -1, (), False, False, None, (
        ('id', (), IcePy._t_string, False, 0),
        ('total', (), IcePy._t_long, False, 0),
        ('current', (), IcePy._t_int, False, 0),
        ('totalLifetime', (), IcePy._t_long, False, 0),
        ('failures', (), IcePy._t_int, False, 0)
    ))
    Metrics._ice_type = _M_IceMX._t_Metrics

    _M_IceMX.Metrics = Metrics
    del Metrics

if 'MetricsFailures' not in _M_IceMX.__dict__:
    _M_IceMX.MetricsFailures = Ice.createTempClass()
    class MetricsFailures(object):
        """
        A structure to keep track of failures associated with a given
        metrics.
        Members:
        id -- The identifier of the metrics object associated to the
        failures.
        failures -- The failures observed for this metrics.
        """
        def __init__(self, id='', failures=None):
            self.id = id
            self.failures = failures

        def __eq__(self, other):
            if other is None:
                return False
            elif not isinstance(other, _M_IceMX.MetricsFailures):
                return NotImplemented
            else:
                if self.id != other.id:
                    return False
                if self.failures != other.failures:
                    return False
                return True

        def __ne__(self, other):
            return not self.__eq__(other)

        def __str__(self):
            return IcePy.stringify(self, _M_IceMX._t_MetricsFailures)

        __repr__ = __str__

    _M_IceMX._t_MetricsFailures = IcePy.defineStruct('::IceMX::MetricsFailures', MetricsFailures, (), (
        ('id', (), IcePy._t_string),
        ('failures', (), _M_IceMX._t_StringIntDict)
    ))

    _M_IceMX.MetricsFailures = MetricsFailures
    del MetricsFailures

if '_t_MetricsFailuresSeq' not in _M_IceMX.__dict__:
    _M_IceMX._t_MetricsFailuresSeq = IcePy.defineSequence('::IceMX::MetricsFailuresSeq', (), _M_IceMX._t_MetricsFailures)

if '_t_MetricsMap' not in _M_IceMX.__dict__:
    _M_IceMX._t_MetricsMap = IcePy.defineSequence('::IceMX::MetricsMap', (), _M_IceMX._t_Metrics)

if '_t_MetricsView' not in _M_IceMX.__dict__:
    _M_IceMX._t_MetricsView = IcePy.defineDictionary('::IceMX::MetricsView', (), IcePy._t_string, _M_IceMX._t_MetricsMap)

if 'UnknownMetricsView' not in _M_IceMX.__dict__:
    _M_IceMX.UnknownMetricsView = Ice.createTempClass()
    class UnknownMetricsView(Ice.UserException):
        """
        Raised if a metrics view cannot be found.
        """
        def __init__(self):
            pass

        def __str__(self):
            return IcePy.stringifyException(self)

        __repr__ = __str__

        _ice_id = '::IceMX::UnknownMetricsView'

    _M_IceMX._t_UnknownMetricsView = IcePy.defineException('::IceMX::UnknownMetricsView', UnknownMetricsView, (), False, None, ())
    UnknownMetricsView._ice_type = _M_IceMX._t_UnknownMetricsView

    _M_IceMX.UnknownMetricsView = UnknownMetricsView
    del UnknownMetricsView

_M_IceMX._t_MetricsAdmin = IcePy.defineValue('::IceMX::MetricsAdmin', Ice.Value, -1, (), False, True, None, ())

if 'MetricsAdminPrx' not in _M_IceMX.__dict__:
    _M_IceMX.MetricsAdminPrx = Ice.createTempClass()
    class MetricsAdminPrx(Ice.ObjectPrx):

        """
        Get the names of enabled and disabled metrics.
        Arguments:
        context -- The request context for the invocation.
        Returns a tuple containing the following:
        _retval -- The name of the enabled views.
        disabledViews -- The names of the disabled views.
        """
        def getMetricsViewNames(self, context=None):
            return _M_IceMX.MetricsAdmin._op_getMetricsViewNames.invoke(self, ((), context))

        """
        Get the names of enabled and disabled metrics.
        Arguments:
        context -- The request context for the invocation.
        Returns: A future object for the invocation.
        """
        def getMetricsViewNamesAsync(self, context=None):
            return _M_IceMX.MetricsAdmin._op_getMetricsViewNames.invokeAsync(self, ((), context))

        """
        Get the names of enabled and disabled metrics.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        context -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getMetricsViewNames(self, _response=None, _ex=None, _sent=None, context=None):
            return _M_IceMX.MetricsAdmin._op_getMetricsViewNames.begin(self, ((), _response, _ex, _sent, context))

        """
        Get the names of enabled and disabled metrics.
        Arguments:
        Returns a tuple containing the following:
        _retval -- The name of the enabled views.
        disabledViews -- The names of the disabled views.
        """
        def end_getMetricsViewNames(self, _r):
            return _M_IceMX.MetricsAdmin._op_getMetricsViewNames.end(self, _r)

        """
        Enables a metrics view.
        Arguments:
        name -- The metrics view name.
        context -- The request context for the invocation.
        Throws:
        UnknownMetricsView -- Raised if the metrics view cannot be found.
        """
        def enableMetricsView(self, name, context=None):
            return _M_IceMX.MetricsAdmin._op_enableMetricsView.invoke(self, ((name, ), context))

        """
        Enables a metrics view.
        Arguments:
        name -- The metrics view name.
        context -- The request context for the invocation.
        Returns: A future object for the invocation.
        """
        def enableMetricsViewAsync(self, name, context=None):
            return _M_IceMX.MetricsAdmin._op_enableMetricsView.invokeAsync(self, ((name, ), context))

        """
        Enables a metrics view.
        Arguments:
        name -- The metrics view name.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        context -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_enableMetricsView(self, name, _response=None, _ex=None, _sent=None, context=None):
            return _M_IceMX.MetricsAdmin._op_enableMetricsView.begin(self, ((name, ), _response, _ex, _sent, context))

        """
        Enables a metrics view.
        Arguments:
        name -- The metrics view name.
        Throws:
        UnknownMetricsView -- Raised if the metrics view cannot be found.
        """
        def end_enableMetricsView(self, _r):
            return _M_IceMX.MetricsAdmin._op_enableMetricsView.end(self, _r)

        """
        Disable a metrics view.
        Arguments:
        name -- The metrics view name.
        context -- The request context for the invocation.
        Throws:
        UnknownMetricsView -- Raised if the metrics view cannot be found.
        """
        def disableMetricsView(self, name, context=None):
            return _M_IceMX.MetricsAdmin._op_disableMetricsView.invoke(self, ((name, ), context))

        """
        Disable a metrics view.
        Arguments:
        name -- The metrics view name.
        context -- The request context for the invocation.
        Returns: A future object for the invocation.
        """
        def disableMetricsViewAsync(self, name, context=None):
            return _M_IceMX.MetricsAdmin._op_disableMetricsView.invokeAsync(self, ((name, ), context))

        """
        Disable a metrics view.
        Arguments:
        name -- The metrics view name.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        context -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_disableMetricsView(self, name, _response=None, _ex=None, _sent=None, context=None):
            return _M_IceMX.MetricsAdmin._op_disableMetricsView.begin(self, ((name, ), _response, _ex, _sent, context))

        """
        Disable a metrics view.
        Arguments:
        name -- The metrics view name.
        Throws:
        UnknownMetricsView -- Raised if the metrics view cannot be found.
        """
        def end_disableMetricsView(self, _r):
            return _M_IceMX.MetricsAdmin._op_disableMetricsView.end(self, _r)

        """
        Get the metrics objects for the given metrics view. This
        returns a dictionnary of metric maps for each metrics class
        configured with the view. The timestamp allows the client to
        compute averages which are not dependent of the invocation
        latency for this operation.
        Arguments:
        view -- The name of the metrics view.
        context -- The request context for the invocation.
        Returns a tuple containing the following:
        _retval -- The metrics view data.
        timestamp -- The local time of the process when the metrics object were retrieved.
        Throws:
        UnknownMetricsView -- Raised if the metrics view cannot be found.
        """
        def getMetricsView(self, view, context=None):
            return _M_IceMX.MetricsAdmin._op_getMetricsView.invoke(self, ((view, ), context))

        """
        Get the metrics objects for the given metrics view. This
        returns a dictionnary of metric maps for each metrics class
        configured with the view. The timestamp allows the client to
        compute averages which are not dependent of the invocation
        latency for this operation.
        Arguments:
        view -- The name of the metrics view.
        context -- The request context for the invocation.
        Returns: A future object for the invocation.
        """
        def getMetricsViewAsync(self, view, context=None):
            return _M_IceMX.MetricsAdmin._op_getMetricsView.invokeAsync(self, ((view, ), context))

        """
        Get the metrics objects for the given metrics view. This
        returns a dictionnary of metric maps for each metrics class
        configured with the view. The timestamp allows the client to
        compute averages which are not dependent of the invocation
        latency for this operation.
        Arguments:
        view -- The name of the metrics view.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        context -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getMetricsView(self, view, _response=None, _ex=None, _sent=None, context=None):
            return _M_IceMX.MetricsAdmin._op_getMetricsView.begin(self, ((view, ), _response, _ex, _sent, context))

        """
        Get the metrics objects for the given metrics view. This
        returns a dictionnary of metric maps for each metrics class
        configured with the view. The timestamp allows the client to
        compute averages which are not dependent of the invocation
        latency for this operation.
        Arguments:
        view -- The name of the metrics view.
        Returns a tuple containing the following:
        _retval -- The metrics view data.
        timestamp -- The local time of the process when the metrics object were retrieved.
        Throws:
        UnknownMetricsView -- Raised if the metrics view cannot be found.
        """
        def end_getMetricsView(self, _r):
            return _M_IceMX.MetricsAdmin._op_getMetricsView.end(self, _r)

        """
        Get the metrics failures associated with the given view and map.
        Arguments:
        view -- The name of the metrics view.
        map -- The name of the metrics map.
        context -- The request context for the invocation.
        Returns: The metrics failures associated with the map.
        Throws:
        UnknownMetricsView -- Raised if the metrics view cannot be found.
        """
        def getMapMetricsFailures(self, view, map, context=None):
            return _M_IceMX.MetricsAdmin._op_getMapMetricsFailures.invoke(self, ((view, map), context))

        """
        Get the metrics failures associated with the given view and map.
        Arguments:
        view -- The name of the metrics view.
        map -- The name of the metrics map.
        context -- The request context for the invocation.
        Returns: A future object for the invocation.
        """
        def getMapMetricsFailuresAsync(self, view, map, context=None):
            return _M_IceMX.MetricsAdmin._op_getMapMetricsFailures.invokeAsync(self, ((view, map), context))

        """
        Get the metrics failures associated with the given view and map.
        Arguments:
        view -- The name of the metrics view.
        map -- The name of the metrics map.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        context -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getMapMetricsFailures(self, view, map, _response=None, _ex=None, _sent=None, context=None):
            return _M_IceMX.MetricsAdmin._op_getMapMetricsFailures.begin(self, ((view, map), _response, _ex, _sent, context))

        """
        Get the metrics failures associated with the given view and map.
        Arguments:
        view -- The name of the metrics view.
        map -- The name of the metrics map.
        Returns: The metrics failures associated with the map.
        Throws:
        UnknownMetricsView -- Raised if the metrics view cannot be found.
        """
        def end_getMapMetricsFailures(self, _r):
            return _M_IceMX.MetricsAdmin._op_getMapMetricsFailures.end(self, _r)

        """
        Get the metrics failure associated for the given metrics.
        Arguments:
        view -- The name of the metrics view.
        map -- The name of the metrics map.
        id -- The ID of the metrics.
        context -- The request context for the invocation.
        Returns: The metrics failures associated with the metrics.
        Throws:
        UnknownMetricsView -- Raised if the metrics view cannot be found.
        """
        def getMetricsFailures(self, view, map, id, context=None):
            return _M_IceMX.MetricsAdmin._op_getMetricsFailures.invoke(self, ((view, map, id), context))

        """
        Get the metrics failure associated for the given metrics.
        Arguments:
        view -- The name of the metrics view.
        map -- The name of the metrics map.
        id -- The ID of the metrics.
        context -- The request context for the invocation.
        Returns: A future object for the invocation.
        """
        def getMetricsFailuresAsync(self, view, map, id, context=None):
            return _M_IceMX.MetricsAdmin._op_getMetricsFailures.invokeAsync(self, ((view, map, id), context))

        """
        Get the metrics failure associated for the given metrics.
        Arguments:
        view -- The name of the metrics view.
        map -- The name of the metrics map.
        id -- The ID of the metrics.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        context -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getMetricsFailures(self, view, map, id, _response=None, _ex=None, _sent=None, context=None):
            return _M_IceMX.MetricsAdmin._op_getMetricsFailures.begin(self, ((view, map, id), _response, _ex, _sent, context))

        """
        Get the metrics failure associated for the given metrics.
        Arguments:
        view -- The name of the metrics view.
        map -- The name of the metrics map.
        id -- The ID of the metrics.
        Returns: The metrics failures associated with the metrics.
        Throws:
        UnknownMetricsView -- Raised if the metrics view cannot be found.
        """
        def end_getMetricsFailures(self, _r):
            return _M_IceMX.MetricsAdmin._op_getMetricsFailures.end(self, _r)

        @staticmethod
        def checkedCast(proxy, facetOrContext=None, context=None):
            return _M_IceMX.MetricsAdminPrx.ice_checkedCast(proxy, '::IceMX::MetricsAdmin', facetOrContext, context)

        @staticmethod
        def uncheckedCast(proxy, facet=None):
            return _M_IceMX.MetricsAdminPrx.ice_uncheckedCast(proxy, facet)

        @staticmethod
        def ice_staticId():
            return '::IceMX::MetricsAdmin'
    _M_IceMX._t_MetricsAdminPrx = IcePy.defineProxy('::IceMX::MetricsAdmin', MetricsAdminPrx)

    _M_IceMX.MetricsAdminPrx = MetricsAdminPrx
    del MetricsAdminPrx

    _M_IceMX.MetricsAdmin = Ice.createTempClass()
    class MetricsAdmin(Ice.Object):

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::IceMX::MetricsAdmin')

        def ice_id(self, current=None):
            return '::IceMX::MetricsAdmin'

        @staticmethod
        def ice_staticId():
            return '::IceMX::MetricsAdmin'

        def getMetricsViewNames(self, current=None):
            """
            Get the names of enabled and disabled metrics.
            Arguments:
            current -- The Current object for the invocation.
            Returns: A future object for the invocation.
            """
            raise NotImplementedError("servant method 'getMetricsViewNames' not implemented")

        def enableMetricsView(self, name, current=None):
            """
            Enables a metrics view.
            Arguments:
            name -- The metrics view name.
            current -- The Current object for the invocation.
            Returns: A future object for the invocation.
            Throws:
            UnknownMetricsView -- Raised if the metrics view cannot be found.
            """
            raise NotImplementedError("servant method 'enableMetricsView' not implemented")

        def disableMetricsView(self, name, current=None):
            """
            Disable a metrics view.
            Arguments:
            name -- The metrics view name.
            current -- The Current object for the invocation.
            Returns: A future object for the invocation.
            Throws:
            UnknownMetricsView -- Raised if the metrics view cannot be found.
            """
            raise NotImplementedError("servant method 'disableMetricsView' not implemented")

        def getMetricsView(self, view, current=None):
            """
            Get the metrics objects for the given metrics view. This
            returns a dictionnary of metric maps for each metrics class
            configured with the view. The timestamp allows the client to
            compute averages which are not dependent of the invocation
            latency for this operation.
            Arguments:
            view -- The name of the metrics view.
            current -- The Current object for the invocation.
            Returns: A future object for the invocation.
            Throws:
            UnknownMetricsView -- Raised if the metrics view cannot be found.
            """
            raise NotImplementedError("servant method 'getMetricsView' not implemented")

        def getMapMetricsFailures(self, view, map, current=None):
            """
            Get the metrics failures associated with the given view and map.
            Arguments:
            view -- The name of the metrics view.
            map -- The name of the metrics map.
            current -- The Current object for the invocation.
            Returns: A future object for the invocation.
            Throws:
            UnknownMetricsView -- Raised if the metrics view cannot be found.
            """
            raise NotImplementedError("servant method 'getMapMetricsFailures' not implemented")

        def getMetricsFailures(self, view, map, id, current=None):
            """
            Get the metrics failure associated for the given metrics.
            Arguments:
            view -- The name of the metrics view.
            map -- The name of the metrics map.
            id -- The ID of the metrics.
            current -- The Current object for the invocation.
            Returns: A future object for the invocation.
            Throws:
            UnknownMetricsView -- Raised if the metrics view cannot be found.
            """
            raise NotImplementedError("servant method 'getMetricsFailures' not implemented")

        def __str__(self):
            return IcePy.stringify(self, _M_IceMX._t_MetricsAdminDisp)

        __repr__ = __str__

    _M_IceMX._t_MetricsAdminDisp = IcePy.defineClass('::IceMX::MetricsAdmin', MetricsAdmin, (), None, ())
    MetricsAdmin._ice_type = _M_IceMX._t_MetricsAdminDisp

    MetricsAdmin._op_getMetricsViewNames = IcePy.Operation('getMetricsViewNames', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, Ice.FormatType.SlicedFormat, (), (), (((), _M_Ice._t_StringSeq, False, 0),), ((), _M_Ice._t_StringSeq, False, 0), ())
    MetricsAdmin._op_enableMetricsView = IcePy.Operation('enableMetricsView', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, Ice.FormatType.SlicedFormat, (), (((), IcePy._t_string, False, 0),), (), None, (_M_IceMX._t_UnknownMetricsView,))
    MetricsAdmin._op_disableMetricsView = IcePy.Operation('disableMetricsView', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, Ice.FormatType.SlicedFormat, (), (((), IcePy._t_string, False, 0),), (), None, (_M_IceMX._t_UnknownMetricsView,))
    MetricsAdmin._op_getMetricsView = IcePy.Operation('getMetricsView', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, Ice.FormatType.SlicedFormat, (), (((), IcePy._t_string, False, 0),), (((), IcePy._t_long, False, 0),), ((), _M_IceMX._t_MetricsView, False, 0), (_M_IceMX._t_UnknownMetricsView,))
    MetricsAdmin._op_getMapMetricsFailures = IcePy.Operation('getMapMetricsFailures', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, Ice.FormatType.SlicedFormat, (), (((), IcePy._t_string, False, 0), ((), IcePy._t_string, False, 0)), (), ((), _M_IceMX._t_MetricsFailuresSeq, False, 0), (_M_IceMX._t_UnknownMetricsView,))
    MetricsAdmin._op_getMetricsFailures = IcePy.Operation('getMetricsFailures', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, Ice.FormatType.SlicedFormat, (), (((), IcePy._t_string, False, 0), ((), IcePy._t_string, False, 0), ((), IcePy._t_string, False, 0)), (), ((), _M_IceMX._t_MetricsFailures, False, 0), (_M_IceMX._t_UnknownMetricsView,))

    _M_IceMX.MetricsAdmin = MetricsAdmin
    del MetricsAdmin

if 'ThreadMetrics' not in _M_IceMX.__dict__:
    _M_IceMX.ThreadMetrics = Ice.createTempClass()
    class ThreadMetrics(_M_IceMX.Metrics):
        """
        Provides information on the number of threads currently in use and
        their activity.
        Members:
        inUseForIO -- The number of threads which are currently performing socket
        read or writes.
        inUseForUser -- The number of threads which are currently calling user code
        (servant dispatch, AMI callbacks, etc).
        inUseForOther -- The number of threads which are currently performing other
        activities. These are all other that are not counted with
        inUseForUser or inUseForIO, such as DNS
        lookups, garbage collection).
        """
        def __init__(self, id='', total=0, current=0, totalLifetime=0, failures=0, inUseForIO=0, inUseForUser=0, inUseForOther=0):
            _M_IceMX.Metrics.__init__(self, id, total, current, totalLifetime, failures)
            self.inUseForIO = inUseForIO
            self.inUseForUser = inUseForUser
            self.inUseForOther = inUseForOther

        def ice_id(self):
            return '::IceMX::ThreadMetrics'

        @staticmethod
        def ice_staticId():
            return '::IceMX::ThreadMetrics'

        def __str__(self):
            return IcePy.stringify(self, _M_IceMX._t_ThreadMetrics)

        __repr__ = __str__

    _M_IceMX._t_ThreadMetrics = IcePy.defineValue('::IceMX::ThreadMetrics', ThreadMetrics, -1, (), False, False, _M_IceMX._t_Metrics, (
        ('inUseForIO', (), IcePy._t_int, False, 0),
        ('inUseForUser', (), IcePy._t_int, False, 0),
        ('inUseForOther', (), IcePy._t_int, False, 0)
    ))
    ThreadMetrics._ice_type = _M_IceMX._t_ThreadMetrics

    _M_IceMX.ThreadMetrics = ThreadMetrics
    del ThreadMetrics

if 'DispatchMetrics' not in _M_IceMX.__dict__:
    _M_IceMX.DispatchMetrics = Ice.createTempClass()
    class DispatchMetrics(_M_IceMX.Metrics):
        """
        Provides information on servant dispatch.
        Members:
        userException -- The number of dispatch that failed with a user exception.
        size -- The size of the dispatch. This corresponds to the size of the
        marshalled input parameters.
        replySize -- The size of the dispatch reply. This corresponds to the size of
        the marshalled output and return parameters.
        """
        def __init__(self, id='', total=0, current=0, totalLifetime=0, failures=0, userException=0, size=0, replySize=0):
            _M_IceMX.Metrics.__init__(self, id, total, current, totalLifetime, failures)
            self.userException = userException
            self.size = size
            self.replySize = replySize

        def ice_id(self):
            return '::IceMX::DispatchMetrics'

        @staticmethod
        def ice_staticId():
            return '::IceMX::DispatchMetrics'

        def __str__(self):
            return IcePy.stringify(self, _M_IceMX._t_DispatchMetrics)

        __repr__ = __str__

    _M_IceMX._t_DispatchMetrics = IcePy.defineValue('::IceMX::DispatchMetrics', DispatchMetrics, -1, (), False, False, _M_IceMX._t_Metrics, (
        ('userException', (), IcePy._t_int, False, 0),
        ('size', (), IcePy._t_long, False, 0),
        ('replySize', (), IcePy._t_long, False, 0)
    ))
    DispatchMetrics._ice_type = _M_IceMX._t_DispatchMetrics

    _M_IceMX.DispatchMetrics = DispatchMetrics
    del DispatchMetrics

if 'ChildInvocationMetrics' not in _M_IceMX.__dict__:
    _M_IceMX.ChildInvocationMetrics = Ice.createTempClass()
    class ChildInvocationMetrics(_M_IceMX.Metrics):
        """
        Provides information on child invocations. A child invocation is
        either remote (sent over an Ice connection) or collocated. An
        invocation can have multiple child invocation if it is
        retried. Child invocation metrics are embedded within
        InvocationMetrics.
        Members:
        size -- The size of the invocation. This corresponds to the size of the
        marshalled input parameters.
        replySize -- The size of the invocation reply. This corresponds to the size
        of the marshalled output and return parameters.
        """
        def __init__(self, id='', total=0, current=0, totalLifetime=0, failures=0, size=0, replySize=0):
            _M_IceMX.Metrics.__init__(self, id, total, current, totalLifetime, failures)
            self.size = size
            self.replySize = replySize

        def ice_id(self):
            return '::IceMX::ChildInvocationMetrics'

        @staticmethod
        def ice_staticId():
            return '::IceMX::ChildInvocationMetrics'

        def __str__(self):
            return IcePy.stringify(self, _M_IceMX._t_ChildInvocationMetrics)

        __repr__ = __str__

    _M_IceMX._t_ChildInvocationMetrics = IcePy.defineValue('::IceMX::ChildInvocationMetrics', ChildInvocationMetrics, -1, (), False, False, _M_IceMX._t_Metrics, (
        ('size', (), IcePy._t_long, False, 0),
        ('replySize', (), IcePy._t_long, False, 0)
    ))
    ChildInvocationMetrics._ice_type = _M_IceMX._t_ChildInvocationMetrics

    _M_IceMX.ChildInvocationMetrics = ChildInvocationMetrics
    del ChildInvocationMetrics

if 'CollocatedMetrics' not in _M_IceMX.__dict__:
    _M_IceMX.CollocatedMetrics = Ice.createTempClass()
    class CollocatedMetrics(_M_IceMX.ChildInvocationMetrics):
        """
        Provides information on invocations that are collocated. Collocated
        metrics are embedded within InvocationMetrics.
        """
        def __init__(self, id='', total=0, current=0, totalLifetime=0, failures=0, size=0, replySize=0):
            _M_IceMX.ChildInvocationMetrics.__init__(self, id, total, current, totalLifetime, failures, size, replySize)

        def ice_id(self):
            return '::IceMX::CollocatedMetrics'

        @staticmethod
        def ice_staticId():
            return '::IceMX::CollocatedMetrics'

        def __str__(self):
            return IcePy.stringify(self, _M_IceMX._t_CollocatedMetrics)

        __repr__ = __str__

    _M_IceMX._t_CollocatedMetrics = IcePy.defineValue('::IceMX::CollocatedMetrics', CollocatedMetrics, -1, (), False, False, _M_IceMX._t_ChildInvocationMetrics, ())
    CollocatedMetrics._ice_type = _M_IceMX._t_CollocatedMetrics

    _M_IceMX.CollocatedMetrics = CollocatedMetrics
    del CollocatedMetrics

if 'RemoteMetrics' not in _M_IceMX.__dict__:
    _M_IceMX.RemoteMetrics = Ice.createTempClass()
    class RemoteMetrics(_M_IceMX.ChildInvocationMetrics):
        """
        Provides information on invocations that are specifically sent over
        Ice connections. Remote metrics are embedded within InvocationMetrics.
        """
        def __init__(self, id='', total=0, current=0, totalLifetime=0, failures=0, size=0, replySize=0):
            _M_IceMX.ChildInvocationMetrics.__init__(self, id, total, current, totalLifetime, failures, size, replySize)

        def ice_id(self):
            return '::IceMX::RemoteMetrics'

        @staticmethod
        def ice_staticId():
            return '::IceMX::RemoteMetrics'

        def __str__(self):
            return IcePy.stringify(self, _M_IceMX._t_RemoteMetrics)

        __repr__ = __str__

    _M_IceMX._t_RemoteMetrics = IcePy.defineValue('::IceMX::RemoteMetrics', RemoteMetrics, -1, (), False, False, _M_IceMX._t_ChildInvocationMetrics, ())
    RemoteMetrics._ice_type = _M_IceMX._t_RemoteMetrics

    _M_IceMX.RemoteMetrics = RemoteMetrics
    del RemoteMetrics

if 'InvocationMetrics' not in _M_IceMX.__dict__:
    _M_IceMX.InvocationMetrics = Ice.createTempClass()
    class InvocationMetrics(_M_IceMX.Metrics):
        """
        Provide measurements for proxy invocations. Proxy invocations can
        either be sent over the wire or be collocated.
        Members:
        retry -- The number of retries for the invocation(s).
        userException -- The number of invocations that failed with a user exception.
        remotes -- The remote invocation metrics map.
        collocated -- The collocated invocation metrics map.
        """
        def __init__(self, id='', total=0, current=0, totalLifetime=0, failures=0, retry=0, userException=0, remotes=None, collocated=None):
            _M_IceMX.Metrics.__init__(self, id, total, current, totalLifetime, failures)
            self.retry = retry
            self.userException = userException
            self.remotes = remotes
            self.collocated = collocated

        def ice_id(self):
            return '::IceMX::InvocationMetrics'

        @staticmethod
        def ice_staticId():
            return '::IceMX::InvocationMetrics'

        def __str__(self):
            return IcePy.stringify(self, _M_IceMX._t_InvocationMetrics)

        __repr__ = __str__

    _M_IceMX._t_InvocationMetrics = IcePy.declareValue('::IceMX::InvocationMetrics')

    _M_IceMX._t_InvocationMetrics = IcePy.defineValue('::IceMX::InvocationMetrics', InvocationMetrics, -1, (), False, False, _M_IceMX._t_Metrics, (
        ('retry', (), IcePy._t_int, False, 0),
        ('userException', (), IcePy._t_int, False, 0),
        ('remotes', (), _M_IceMX._t_MetricsMap, False, 0),
        ('collocated', (), _M_IceMX._t_MetricsMap, False, 0)
    ))
    InvocationMetrics._ice_type = _M_IceMX._t_InvocationMetrics

    _M_IceMX.InvocationMetrics = InvocationMetrics
    del InvocationMetrics

if 'ConnectionMetrics' not in _M_IceMX.__dict__:
    _M_IceMX.ConnectionMetrics = Ice.createTempClass()
    class ConnectionMetrics(_M_IceMX.Metrics):
        """
        Provides information on the data sent and received over Ice
        connections.
        Members:
        receivedBytes -- The number of bytes received by the connection.
        sentBytes -- The number of bytes sent by the connection.
        """
        def __init__(self, id='', total=0, current=0, totalLifetime=0, failures=0, receivedBytes=0, sentBytes=0):
            _M_IceMX.Metrics.__init__(self, id, total, current, totalLifetime, failures)
            self.receivedBytes = receivedBytes
            self.sentBytes = sentBytes

        def ice_id(self):
            return '::IceMX::ConnectionMetrics'

        @staticmethod
        def ice_staticId():
            return '::IceMX::ConnectionMetrics'

        def __str__(self):
            return IcePy.stringify(self, _M_IceMX._t_ConnectionMetrics)

        __repr__ = __str__

    _M_IceMX._t_ConnectionMetrics = IcePy.defineValue('::IceMX::ConnectionMetrics', ConnectionMetrics, -1, (), False, False, _M_IceMX._t_Metrics, (
        ('receivedBytes', (), IcePy._t_long, False, 0),
        ('sentBytes', (), IcePy._t_long, False, 0)
    ))
    ConnectionMetrics._ice_type = _M_IceMX._t_ConnectionMetrics

    _M_IceMX.ConnectionMetrics = ConnectionMetrics
    del ConnectionMetrics

# End of module IceMX

Ice.sliceChecksums["::IceMX::ChildInvocationMetrics"] = "e8d7639c4944abd2b46e1676712e3914"
Ice.sliceChecksums["::IceMX::CollocatedMetrics"] = "3249804089f8e0882d12276c8e6d4165"
Ice.sliceChecksums["::IceMX::ConnectionMetrics"] = "ff705e359a88ff9bc35ab28283c8e29"
Ice.sliceChecksums["::IceMX::DispatchMetrics"] = "e583ad5ced38253d9ff59929af294ba"
Ice.sliceChecksums["::IceMX::InvocationMetrics"] = "64216b477afc9431249a60ae5913f8"
Ice.sliceChecksums["::IceMX::Metrics"] = "94c8036c48f8749deb5f33a364ecee"
Ice.sliceChecksums["::IceMX::MetricsAdmin"] = "88697f8371c057b7177760281b33a5"
Ice.sliceChecksums["::IceMX::MetricsFailures"] = "19ecb6e915befa597421fa4c92a1ac"
Ice.sliceChecksums["::IceMX::MetricsFailuresSeq"] = "a62163545a1a5a54ade5d7d826aac8bd"
Ice.sliceChecksums["::IceMX::MetricsMap"] = "22667dd9415a83de99a17cf19f63975"
Ice.sliceChecksums["::IceMX::MetricsView"] = "c8c150b17e594dea2e3c7c2d4738b6"
Ice.sliceChecksums["::IceMX::RemoteMetrics"] = "1dda65bb856a2c2fcaf32ea2b40682f"
Ice.sliceChecksums["::IceMX::StringIntDict"] = "e6c7aa764386f6528aa38c89cbff5dd4"
Ice.sliceChecksums["::IceMX::ThreadMetrics"] = "54ca6eb235a9769ade47e8ae200ff18"
Ice.sliceChecksums["::IceMX::UnknownMetricsView"] = "18de0c2a8812deb6facfd521d84ba6"



# -*- coding: utf-8 -*-
#
# Copyright (c) ZeroC, Inc. All rights reserved.
#
#
# Ice version 3.7.6
#
# <auto-generated>
#
# Generated from file `EndpointF.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy

# Start of module Ice
_M_Ice = Ice.openModule('Ice')
__name__ = 'Ice'

if 'EndpointInfo' not in _M_Ice.__dict__:
    _M_Ice._t_EndpointInfo = IcePy.declareValue('::Ice::EndpointInfo')

if 'IPEndpointInfo' not in _M_Ice.__dict__:
    _M_Ice._t_IPEndpointInfo = IcePy.declareValue('::Ice::IPEndpointInfo')

if 'TCPEndpointInfo' not in _M_Ice.__dict__:
    _M_Ice._t_TCPEndpointInfo = IcePy.declareValue('::Ice::TCPEndpointInfo')

if 'UDPEndpointInfo' not in _M_Ice.__dict__:
    _M_Ice._t_UDPEndpointInfo = IcePy.declareValue('::Ice::UDPEndpointInfo')

if 'WSEndpointInfo' not in _M_Ice.__dict__:
    _M_Ice._t_WSEndpointInfo = IcePy.declareValue('::Ice::WSEndpointInfo')

if 'Endpoint' not in _M_Ice.__dict__:
    _M_Ice._t_Endpoint = IcePy.declareValue('::Ice::Endpoint')

if '_t_EndpointSeq' not in _M_Ice.__dict__:
    _M_Ice._t_EndpointSeq = IcePy.defineSequence('::Ice::EndpointSeq', (), _M_Ice._t_Endpoint)

# End of module Ice



# -*- coding: utf-8 -*-
#
# Copyright (c) ZeroC, Inc. All rights reserved.
#
#
# Ice version 3.7.6
#
# <auto-generated>
#
# Generated from file `Identity.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy

# Start of module Ice
_M_Ice = Ice.openModule('Ice')
__name__ = 'Ice'

if 'Identity' not in _M_Ice.__dict__:
    _M_Ice.Identity = Ice.createTempClass()
    class Identity(object):
        """
        The identity of an Ice object. In a proxy, an empty Identity#name denotes a nil
        proxy. An identity with an empty Identity#name and a non-empty Identity#category
        is illegal. You cannot add a servant with an empty name to the Active Servant Map.
        Members:
        name -- The name of the Ice object.
        category -- The Ice object category.
        """
        def __init__(self, name='', category=''):
            self.name = name
            self.category = category

        def __hash__(self):
            _h = 0
            _h = 5 * _h + Ice.getHash(self.name)
            _h = 5 * _h + Ice.getHash(self.category)
            return _h % 0x7fffffff

        def __compare(self, other):
            if other is None:
                return 1
            elif not isinstance(other, _M_Ice.Identity):
                return NotImplemented
            else:
                if self.name is None or other.name is None:
                    if self.name != other.name:
                        return (-1 if self.name is None else 1)
                else:
                    if self.name < other.name:
                        return -1
                    elif self.name > other.name:
                        return 1
                if self.category is None or other.category is None:
                    if self.category != other.category:
                        return (-1 if self.category is None else 1)
                else:
                    if self.category < other.category:
                        return -1
                    elif self.category > other.category:
                        return 1
                return 0

        def __lt__(self, other):
            r = self.__compare(other)
            if r is NotImplemented:
                return r
            else:
                return r < 0

        def __le__(self, other):
            r = self.__compare(other)
            if r is NotImplemented:
                return r
            else:
                return r <= 0

        def __gt__(self, other):
            r = self.__compare(other)
            if r is NotImplemented:
                return r
            else:
                return r > 0

        def __ge__(self, other):
            r = self.__compare(other)
            if r is NotImplemented:
                return r
            else:
                return r >= 0

        def __eq__(self, other):
            r = self.__compare(other)
            if r is NotImplemented:
                return r
            else:
                return r == 0

        def __ne__(self, other):
            r = self.__compare(other)
            if r is NotImplemented:
                return r
            else:
                return r != 0

        def __str__(self):
            return IcePy.stringify(self, _M_Ice._t_Identity)

        __repr__ = __str__

    _M_Ice._t_Identity = IcePy.defineStruct('::Ice::Identity', Identity, (), (
        ('name', (), IcePy._t_string),
        ('category', (), IcePy._t_string)
    ))

    _M_Ice.Identity = Identity
    del Identity

if '_t_ObjectDict' not in _M_Ice.__dict__:
    _M_Ice._t_ObjectDict = IcePy.defineDictionary('::Ice::ObjectDict', (), _M_Ice._t_Identity, IcePy._t_Value)

if '_t_IdentitySeq' not in _M_Ice.__dict__:
    _M_Ice._t_IdentitySeq = IcePy.defineSequence('::Ice::IdentitySeq', (), _M_Ice._t_Identity)

# End of module Ice

Ice.sliceChecksums["::Ice::Identity"] = "a0c37867a69924a9fcc4bdf078f06da"
Ice.sliceChecksums["::Ice::IdentitySeq"] = "90afa299fd9e7d7a17268cc22b5ca42"



# -*- coding: utf-8 -*-
#
# Copyright (c) ZeroC, Inc. All rights reserved.
#
#
# Ice version 3.7.6
#
# <auto-generated>
#
# Generated from file `ValueFactory.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy

# Start of module Ice
_M_Ice = Ice.openModule('Ice')
__name__ = 'Ice'

if 'ValueFactory' not in _M_Ice.__dict__:
    _M_Ice.ValueFactory = Ice.createTempClass()
    class ValueFactory(object):
        """
        A factory for values. Value factories are used in several
        places, such as when Ice receives a class instance and
        when Freeze restores a persistent value. Value factories
        must be implemented by the application writer and registered
        with the communicator.
        """
        def __init__(self):
            if Ice.getType(self) == _M_Ice.ValueFactory:
                raise RuntimeError('Ice.ValueFactory is an abstract class')

        def create(self, type):
            """
            Create a new value for a given value type. The type is the
            absolute Slice type id, i.e., the id relative to the
            unnamed top-level Slice module. For example, the absolute
            Slice type id for an interface Bar in the module
            Foo is "::Foo::Bar".
            Note that the leading "::" is required.
            Arguments:
            type -- The value type.
            Returns: The value created for the given type, or nil if the factory is unable to create the value.
            """
            raise NotImplementedError("method 'create' not implemented")

        def __str__(self):
            return IcePy.stringify(self, _M_Ice._t_ValueFactory)

        __repr__ = __str__

    _M_Ice._t_ValueFactory = IcePy.defineValue('::Ice::ValueFactory', ValueFactory, -1, (), False, True, None, ())
    ValueFactory._ice_type = _M_Ice._t_ValueFactory

    _M_Ice.ValueFactory = ValueFactory
    del ValueFactory

if 'ValueFactoryManager' not in _M_Ice.__dict__:
    _M_Ice.ValueFactoryManager = Ice.createTempClass()
    class ValueFactoryManager(object):
        """
        A value factory manager maintains a collection of value factories.
        An application can supply a custom implementation during communicator
        initialization, otherwise Ice provides a default implementation.
        """
        def __init__(self):
            if Ice.getType(self) == _M_Ice.ValueFactoryManager:
                raise RuntimeError('Ice.ValueFactoryManager is an abstract class')

        def add(self, factory, id):
            """
            Add a value factory. Attempting to add a factory with an id for
            which a factory is already registered throws AlreadyRegisteredException.
            When unmarshaling an Ice value, the Ice run time reads the
            most-derived type id off the wire and attempts to create an
            instance of the type using a factory. If no instance is created,
            either because no factory was found, or because all factories
            returned nil, the behavior of the Ice run time depends on the
            format with which the value was marshaled:
            If the value uses the "sliced" format, Ice ascends the class
            hierarchy until it finds a type that is recognized by a factory,
            or it reaches the least-derived type. If no factory is found that
            can create an instance, the run time throws NoValueFactoryException.
            If the value uses the "compact" format, Ice immediately raises
            NoValueFactoryException.
            The following order is used to locate a factory for a type:
            The Ice run-time looks for a factory registered
            specifically for the type.
            If no instance has been created, the Ice run-time looks
            for the default factory, which is registered with an empty type id.
            If no instance has been created by any of the preceding
            steps, the Ice run-time looks for a factory that may have been
            statically generated by the language mapping for non-abstract classes.
            Arguments:
            factory -- The factory to add.
            id -- The type id for which the factory can create instances, or an empty string for the default factory.
            """
            raise NotImplementedError("method 'add' not implemented")

        def find(self, id):
            """
            Find an value factory registered with this communicator.
            Arguments:
            id -- The type id for which the factory can create instances, or an empty string for the default factory.
            Returns: The value factory, or null if no value factory was found for the given id.
            """
            raise NotImplementedError("method 'find' not implemented")

        def __str__(self):
            return IcePy.stringify(self, _M_Ice._t_ValueFactoryManager)

        __repr__ = __str__

    _M_Ice._t_ValueFactoryManager = IcePy.defineValue('::Ice::ValueFactoryManager', ValueFactoryManager, -1, (), False, True, None, ())
    ValueFactoryManager._ice_type = _M_Ice._t_ValueFactoryManager

    _M_Ice.ValueFactoryManager = ValueFactoryManager
    del ValueFactoryManager

# End of module Ice



# Generated by slice2py - DO NOT EDIT!
#

import Ice
Ice.updateModule("IceGrid")

# Modules:
import IceGrid.Descriptor_ice
import IceGrid.PluginFacade_ice
import IceGrid.UserAccountMapper_ice
import IceGrid.Session_ice
import IceGrid.Admin_ice
import IceGrid.FileParser_ice
import IceGrid.Registry_ice
import IceGrid.Exception_ice

# Submodules:



# -*- coding: utf-8 -*-
#
# Copyright (c) ZeroC, Inc. All rights reserved.
#
#
# Ice version 3.7.6
#
# <auto-generated>
#
# Generated from file `FacetMap.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy

# Start of module Ice
_M_Ice = Ice.openModule('Ice')
__name__ = 'Ice'

if '_t_FacetMap' not in _M_Ice.__dict__:
    _M_Ice._t_FacetMap = IcePy.defineDictionary('::Ice::FacetMap', (), IcePy._t_string, IcePy._t_Value)

# End of module Ice



# -*- coding: utf-8 -*-
#
# Copyright (c) ZeroC, Inc. All rights reserved.
#
#
# Ice version 3.7.6
#
# <auto-generated>
#
# Generated from file `IceBox.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import Ice.BuiltinSequences_ice
import Ice.CommunicatorF_ice
import Ice.PropertiesF_ice
import Ice.SliceChecksumDict_ice

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Start of module IceBox
_M_IceBox = Ice.openModule('IceBox')
__name__ = 'IceBox'
_M_IceBox.__doc__ = """
IceBox is an application server specifically for Ice
applications. IceBox can easily run and administer Ice services
that are dynamically loaded as a DLL, shared library, or Java
class.
"""

if 'FailureException' not in _M_IceBox.__dict__:
    _M_IceBox.FailureException = Ice.createTempClass()
    class FailureException(Ice.LocalException):
        """
        This exception is a general failure notification. It is thrown
        for errors such as a service encountering an error during
        initialization, or the service manager being unable
        to load a service executable.
        Members:
        reason -- The reason for the failure.
        """
        def __init__(self, reason=''):
            self.reason = reason

        def __str__(self):
            return IcePy.stringifyException(self)

        __repr__ = __str__

        _ice_id = '::IceBox::FailureException'

    _M_IceBox._t_FailureException = IcePy.defineException('::IceBox::FailureException', FailureException, (), False, None, (('reason', (), IcePy._t_string, False, 0),))
    FailureException._ice_type = _M_IceBox._t_FailureException

    _M_IceBox.FailureException = FailureException
    del FailureException

if 'AlreadyStartedException' not in _M_IceBox.__dict__:
    _M_IceBox.AlreadyStartedException = Ice.createTempClass()
    class AlreadyStartedException(Ice.UserException):
        """
        This exception is thrown if an attempt is made to start an
        already-started service.
        """
        def __init__(self):
            pass

        def __str__(self):
            return IcePy.stringifyException(self)

        __repr__ = __str__

        _ice_id = '::IceBox::AlreadyStartedException'

    _M_IceBox._t_AlreadyStartedException = IcePy.defineException('::IceBox::AlreadyStartedException', AlreadyStartedException, (), False, None, ())
    AlreadyStartedException._ice_type = _M_IceBox._t_AlreadyStartedException

    _M_IceBox.AlreadyStartedException = AlreadyStartedException
    del AlreadyStartedException

if 'AlreadyStoppedException' not in _M_IceBox.__dict__:
    _M_IceBox.AlreadyStoppedException = Ice.createTempClass()
    class AlreadyStoppedException(Ice.UserException):
        """
        This exception is thrown if an attempt is made to stop an
        already-stopped service.
        """
        def __init__(self):
            pass

        def __str__(self):
            return IcePy.stringifyException(self)

        __repr__ = __str__

        _ice_id = '::IceBox::AlreadyStoppedException'

    _M_IceBox._t_AlreadyStoppedException = IcePy.defineException('::IceBox::AlreadyStoppedException', AlreadyStoppedException, (), False, None, ())
    AlreadyStoppedException._ice_type = _M_IceBox._t_AlreadyStoppedException

    _M_IceBox.AlreadyStoppedException = AlreadyStoppedException
    del AlreadyStoppedException

if 'NoSuchServiceException' not in _M_IceBox.__dict__:
    _M_IceBox.NoSuchServiceException = Ice.createTempClass()
    class NoSuchServiceException(Ice.UserException):
        """
        This exception is thrown if a service name does not refer
        to an existing service.
        """
        def __init__(self):
            pass

        def __str__(self):
            return IcePy.stringifyException(self)

        __repr__ = __str__

        _ice_id = '::IceBox::NoSuchServiceException'

    _M_IceBox._t_NoSuchServiceException = IcePy.defineException('::IceBox::NoSuchServiceException', NoSuchServiceException, (), False, None, ())
    NoSuchServiceException._ice_type = _M_IceBox._t_NoSuchServiceException

    _M_IceBox.NoSuchServiceException = NoSuchServiceException
    del NoSuchServiceException

if 'Service' not in _M_IceBox.__dict__:
    _M_IceBox.Service = Ice.createTempClass()
    class Service(object):
        """
        An application service managed by a ServiceManager.
        """
        def __init__(self):
            if Ice.getType(self) == _M_IceBox.Service:
                raise RuntimeError('IceBox.Service is an abstract class')

        def start(self, name, communicator, args):
            """
            Start the service. The given communicator is created by the
            ServiceManager for use by the service. This communicator may
            also be used by other services, depending on the service
            configuration.
            The ServiceManager owns this communicator, and is
            responsible for destroying it.
            Arguments:
            name -- The service's name, as determined by the configuration.
            communicator -- A communicator for use by the service.
            args -- The service arguments that were not converted into properties.
            Throws:
            FailureException -- Raised if start failed.
            """
            raise NotImplementedError("method 'start' not implemented")

        def stop(self):
            """
            Stop the service.
            """
            raise NotImplementedError("method 'stop' not implemented")

        def __str__(self):
            return IcePy.stringify(self, _M_IceBox._t_Service)

        __repr__ = __str__

    _M_IceBox._t_Service = IcePy.defineValue('::IceBox::Service', Service, -1, (), False, True, None, ())
    Service._ice_type = _M_IceBox._t_Service

    _M_IceBox.Service = Service
    del Service

_M_IceBox._t_ServiceObserver = IcePy.defineValue('::IceBox::ServiceObserver', Ice.Value, -1, (), False, True, None, ())

if 'ServiceObserverPrx' not in _M_IceBox.__dict__:
    _M_IceBox.ServiceObserverPrx = Ice.createTempClass()
    class ServiceObserverPrx(Ice.ObjectPrx):

        """
        Receives the names of the services that were started.
        Arguments:
        services -- The names of the services.
        context -- The request context for the invocation.
        """
        def servicesStarted(self, services, context=None):
            return _M_IceBox.ServiceObserver._op_servicesStarted.invoke(self, ((services, ), context))

        """
        Receives the names of the services that were started.
        Arguments:
        services -- The names of the services.
        context -- The request context for the invocation.
        Returns: A future object for the invocation.
        """
        def servicesStartedAsync(self, services, context=None):
            return _M_IceBox.ServiceObserver._op_servicesStarted.invokeAsync(self, ((services, ), context))

        """
        Receives the names of the services that were started.
        Arguments:
        services -- The names of the services.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        context -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_servicesStarted(self, services, _response=None, _ex=None, _sent=None, context=None):
            return _M_IceBox.ServiceObserver._op_servicesStarted.begin(self, ((services, ), _response, _ex, _sent, context))

        """
        Receives the names of the services that were started.
        Arguments:
        services -- The names of the services.
        """
        def end_servicesStarted(self, _r):
            return _M_IceBox.ServiceObserver._op_servicesStarted.end(self, _r)

        """
        Receives the names of the services that were stopped.
        Arguments:
        services -- The names of the services.
        context -- The request context for the invocation.
        """
        def servicesStopped(self, services, context=None):
            return _M_IceBox.ServiceObserver._op_servicesStopped.invoke(self, ((services, ), context))

        """
        Receives the names of the services that were stopped.
        Arguments:
        services -- The names of the services.
        context -- The request context for the invocation.
        Returns: A future object for the invocation.
        """
        def servicesStoppedAsync(self, services, context=None):
            return _M_IceBox.ServiceObserver._op_servicesStopped.invokeAsync(self, ((services, ), context))

        """
        Receives the names of the services that were stopped.
        Arguments:
        services -- The names of the services.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        context -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_servicesStopped(self, services, _response=None, _ex=None, _sent=None, context=None):
            return _M_IceBox.ServiceObserver._op_servicesStopped.begin(self, ((services, ), _response, _ex, _sent, context))

        """
        Receives the names of the services that were stopped.
        Arguments:
        services -- The names of the services.
        """
        def end_servicesStopped(self, _r):
            return _M_IceBox.ServiceObserver._op_servicesStopped.end(self, _r)

        @staticmethod
        def checkedCast(proxy, facetOrContext=None, context=None):
            return _M_IceBox.ServiceObserverPrx.ice_checkedCast(proxy, '::IceBox::ServiceObserver', facetOrContext, context)

        @staticmethod
        def uncheckedCast(proxy, facet=None):
            return _M_IceBox.ServiceObserverPrx.ice_uncheckedCast(proxy, facet)

        @staticmethod
        def ice_staticId():
            return '::IceBox::ServiceObserver'
    _M_IceBox._t_ServiceObserverPrx = IcePy.defineProxy('::IceBox::ServiceObserver', ServiceObserverPrx)

    _M_IceBox.ServiceObserverPrx = ServiceObserverPrx
    del ServiceObserverPrx

    _M_IceBox.ServiceObserver = Ice.createTempClass()
    class ServiceObserver(Ice.Object):

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::IceBox::ServiceObserver')

        def ice_id(self, current=None):
            return '::IceBox::ServiceObserver'

        @staticmethod
        def ice_staticId():
            return '::IceBox::ServiceObserver'

        def servicesStarted(self, services, current=None):
            """
            Receives the names of the services that were started.
            Arguments:
            services -- The names of the services.
            current -- The Current object for the invocation.
            Returns: A future object for the invocation.
            """
            raise NotImplementedError("servant method 'servicesStarted' not implemented")

        def servicesStopped(self, services, current=None):
            """
            Receives the names of the services that were stopped.
            Arguments:
            services -- The names of the services.
            current -- The Current object for the invocation.
            Returns: A future object for the invocation.
            """
            raise NotImplementedError("servant method 'servicesStopped' not implemented")

        def __str__(self):
            return IcePy.stringify(self, _M_IceBox._t_ServiceObserverDisp)

        __repr__ = __str__

    _M_IceBox._t_ServiceObserverDisp = IcePy.defineClass('::IceBox::ServiceObserver', ServiceObserver, (), None, ())
    ServiceObserver._ice_type = _M_IceBox._t_ServiceObserverDisp

    ServiceObserver._op_servicesStarted = IcePy.Operation('servicesStarted', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_Ice._t_StringSeq, False, 0),), (), None, ())
    ServiceObserver._op_servicesStopped = IcePy.Operation('servicesStopped', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_Ice._t_StringSeq, False, 0),), (), None, ())

    _M_IceBox.ServiceObserver = ServiceObserver
    del ServiceObserver

_M_IceBox._t_ServiceManager = IcePy.defineValue('::IceBox::ServiceManager', Ice.Value, -1, (), False, True, None, ())

if 'ServiceManagerPrx' not in _M_IceBox.__dict__:
    _M_IceBox.ServiceManagerPrx = Ice.createTempClass()
    class ServiceManagerPrx(Ice.ObjectPrx):

        """
        Returns the checksums for the IceBox Slice definitions.
        Arguments:
        context -- The request context for the invocation.
        Returns: A dictionary mapping Slice type ids to their checksums.
        """
        def getSliceChecksums(self, context=None):
            return _M_IceBox.ServiceManager._op_getSliceChecksums.invoke(self, ((), context))

        """
        Returns the checksums for the IceBox Slice definitions.
        Arguments:
        context -- The request context for the invocation.
        Returns: A future object for the invocation.
        """
        def getSliceChecksumsAsync(self, context=None):
            return _M_IceBox.ServiceManager._op_getSliceChecksums.invokeAsync(self, ((), context))

        """
        Returns the checksums for the IceBox Slice definitions.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        context -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getSliceChecksums(self, _response=None, _ex=None, _sent=None, context=None):
            return _M_IceBox.ServiceManager._op_getSliceChecksums.begin(self, ((), _response, _ex, _sent, context))

        """
        Returns the checksums for the IceBox Slice definitions.
        Arguments:
        Returns: A dictionary mapping Slice type ids to their checksums.
        """
        def end_getSliceChecksums(self, _r):
            return _M_IceBox.ServiceManager._op_getSliceChecksums.end(self, _r)

        """
        Start an individual service.
        Arguments:
        service -- The service name.
        context -- The request context for the invocation.
        Throws:
        AlreadyStartedException -- If the service is already running.
        NoSuchServiceException -- If no service could be found with the given name.
        """
        def startService(self, service, context=None):
            return _M_IceBox.ServiceManager._op_startService.invoke(self, ((service, ), context))

        """
        Start an individual service.
        Arguments:
        service -- The service name.
        context -- The request context for the invocation.
        Returns: A future object for the invocation.
        """
        def startServiceAsync(self, service, context=None):
            return _M_IceBox.ServiceManager._op_startService.invokeAsync(self, ((service, ), context))

        """
        Start an individual service.
        Arguments:
        service -- The service name.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        context -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_startService(self, service, _response=None, _ex=None, _sent=None, context=None):
            return _M_IceBox.ServiceManager._op_startService.begin(self, ((service, ), _response, _ex, _sent, context))

        """
        Start an individual service.
        Arguments:
        service -- The service name.
        Throws:
        AlreadyStartedException -- If the service is already running.
        NoSuchServiceException -- If no service could be found with the given name.
        """
        def end_startService(self, _r):
            return _M_IceBox.ServiceManager._op_startService.end(self, _r)

        """
        Stop an individual service.
        Arguments:
        service -- The service name.
        context -- The request context for the invocation.
        Throws:
        AlreadyStoppedException -- If the service is already stopped.
        NoSuchServiceException -- If no service could be found with the given name.
        """
        def stopService(self, service, context=None):
            return _M_IceBox.ServiceManager._op_stopService.invoke(self, ((service, ), context))

        """
        Stop an individual service.
        Arguments:
        service -- The service name.
        context -- The request context for the invocation.
        Returns: A future object for the invocation.
        """
        def stopServiceAsync(self, service, context=None):
            return _M_IceBox.ServiceManager._op_stopService.invokeAsync(self, ((service, ), context))

        """
        Stop an individual service.
        Arguments:
        service -- The service name.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        context -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_stopService(self, service, _response=None, _ex=None, _sent=None, context=None):
            return _M_IceBox.ServiceManager._op_stopService.begin(self, ((service, ), _response, _ex, _sent, context))

        """
        Stop an individual service.
        Arguments:
        service -- The service name.
        Throws:
        AlreadyStoppedException -- If the service is already stopped.
        NoSuchServiceException -- If no service could be found with the given name.
        """
        def end_stopService(self, _r):
            return _M_IceBox.ServiceManager._op_stopService.end(self, _r)

        """
        Registers a new observer with the ServiceManager.
        Arguments:
        observer -- The new observer
        context -- The request context for the invocation.
        """
        def addObserver(self, observer, context=None):
            return _M_IceBox.ServiceManager._op_addObserver.invoke(self, ((observer, ), context))

        """
        Registers a new observer with the ServiceManager.
        Arguments:
        observer -- The new observer
        context -- The request context for the invocation.
        Returns: A future object for the invocation.
        """
        def addObserverAsync(self, observer, context=None):
            return _M_IceBox.ServiceManager._op_addObserver.invokeAsync(self, ((observer, ), context))

        """
        Registers a new observer with the ServiceManager.
        Arguments:
        observer -- The new observer
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        context -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_addObserver(self, observer, _response=None, _ex=None, _sent=None, context=None):
            return _M_IceBox.ServiceManager._op_addObserver.begin(self, ((observer, ), _response, _ex, _sent, context))

        """
        Registers a new observer with the ServiceManager.
        Arguments:
        observer -- The new observer
        """
        def end_addObserver(self, _r):
            return _M_IceBox.ServiceManager._op_addObserver.end(self, _r)

        """
        Shut down all services. This causes stop to be invoked on all configured services.
        Arguments:
        context -- The request context for the invocation.
        """
        def shutdown(self, context=None):
            return _M_IceBox.ServiceManager._op_shutdown.invoke(self, ((), context))

        """
        Shut down all services. This causes stop to be invoked on all configured services.
        Arguments:
        context -- The request context for the invocation.
        Returns: A future object for the invocation.
        """
        def shutdownAsync(self, context=None):
            return _M_IceBox.ServiceManager._op_shutdown.invokeAsync(self, ((), context))

        """
        Shut down all services. This causes stop to be invoked on all configured services.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        context -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_shutdown(self, _response=None, _ex=None, _sent=None, context=None):
            return _M_IceBox.ServiceManager._op_shutdown.begin(self, ((), _response, _ex, _sent, context))

        """
        Shut down all services. This causes stop to be invoked on all configured services.
        Arguments:
        """
        def end_shutdown(self, _r):
            return _M_IceBox.ServiceManager._op_shutdown.end(self, _r)

        @staticmethod
        def checkedCast(proxy, facetOrContext=None, context=None):
            return _M_IceBox.ServiceManagerPrx.ice_checkedCast(proxy, '::IceBox::ServiceManager', facetOrContext, context)

        @staticmethod
        def uncheckedCast(proxy, facet=None):
            return _M_IceBox.ServiceManagerPrx.ice_uncheckedCast(proxy, facet)

        @staticmethod
        def ice_staticId():
            return '::IceBox::ServiceManager'
    _M_IceBox._t_ServiceManagerPrx = IcePy.defineProxy('::IceBox::ServiceManager', ServiceManagerPrx)

    _M_IceBox.ServiceManagerPrx = ServiceManagerPrx
    del ServiceManagerPrx

    _M_IceBox.ServiceManager = Ice.createTempClass()
    class ServiceManager(Ice.Object):

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::IceBox::ServiceManager')

        def ice_id(self, current=None):
            return '::IceBox::ServiceManager'

        @staticmethod
        def ice_staticId():
            return '::IceBox::ServiceManager'

        def getSliceChecksums(self, current=None):
            """
            Returns the checksums for the IceBox Slice definitions.
            Arguments:
            current -- The Current object for the invocation.
            Returns: A future object for the invocation.
            """
            raise NotImplementedError("servant method 'getSliceChecksums' not implemented")

        def startService(self, service, current=None):
            """
            Start an individual service.
            Arguments:
            service -- The service name.
            current -- The Current object for the invocation.
            Returns: A future object for the invocation.
            Throws:
            AlreadyStartedException -- If the service is already running.
            NoSuchServiceException -- If no service could be found with the given name.
            """
            raise NotImplementedError("servant method 'startService' not implemented")

        def stopService(self, service, current=None):
            """
            Stop an individual service.
            Arguments:
            service -- The service name.
            current -- The Current object for the invocation.
            Returns: A future object for the invocation.
            Throws:
            AlreadyStoppedException -- If the service is already stopped.
            NoSuchServiceException -- If no service could be found with the given name.
            """
            raise NotImplementedError("servant method 'stopService' not implemented")

        def addObserver(self, observer, current=None):
            """
            Registers a new observer with the ServiceManager.
            Arguments:
            observer -- The new observer
            current -- The Current object for the invocation.
            Returns: A future object for the invocation.
            """
            raise NotImplementedError("servant method 'addObserver' not implemented")

        def shutdown(self, current=None):
            """
            Shut down all services. This causes stop to be invoked on all configured services.
            Arguments:
            current -- The Current object for the invocation.
            Returns: A future object for the invocation.
            """
            raise NotImplementedError("servant method 'shutdown' not implemented")

        def __str__(self):
            return IcePy.stringify(self, _M_IceBox._t_ServiceManagerDisp)

        __repr__ = __str__

    _M_IceBox._t_ServiceManagerDisp = IcePy.defineClass('::IceBox::ServiceManager', ServiceManager, (), None, ())
    ServiceManager._ice_type = _M_IceBox._t_ServiceManagerDisp

    ServiceManager._op_getSliceChecksums = IcePy.Operation('getSliceChecksums', Ice.OperationMode.Idempotent, Ice.OperationMode.Nonmutating, False, None, (), (), (), ((), _M_Ice._t_SliceChecksumDict, False, 0), ())
    ServiceManager._op_startService = IcePy.Operation('startService', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_string, False, 0),), (), None, (_M_IceBox._t_AlreadyStartedException, _M_IceBox._t_NoSuchServiceException))
    ServiceManager._op_stopService = IcePy.Operation('stopService', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_string, False, 0),), (), None, (_M_IceBox._t_AlreadyStoppedException, _M_IceBox._t_NoSuchServiceException))
    ServiceManager._op_addObserver = IcePy.Operation('addObserver', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_IceBox._t_ServiceObserverPrx, False, 0),), (), None, ())
    ServiceManager._op_shutdown = IcePy.Operation('shutdown', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())

    _M_IceBox.ServiceManager = ServiceManager
    del ServiceManager

# End of module IceBox

Ice.sliceChecksums["::IceBox::AlreadyStartedException"] = "d5b097af3221b37482d5f175502abf62"
Ice.sliceChecksums["::IceBox::AlreadyStoppedException"] = "281d493a84d674b3a4335d6afc2c16"
Ice.sliceChecksums["::IceBox::NoSuchServiceException"] = "5957f1c582d9aebad557cbdb7820d4"
Ice.sliceChecksums["::IceBox::ServiceManager"] = "df3a42670c3ce4ef67d6125a5d04d4c"
Ice.sliceChecksums["::IceBox::ServiceObserver"] = "f657781cda7438532a6c33e95988479c"



# -*- coding: utf-8 -*-
#
# Copyright (c) ZeroC, Inc. All rights reserved.
#
#
# Ice version 3.7.6
#
# <auto-generated>
#
# Generated from file `ObjectFactory.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy

# Start of module Ice
_M_Ice = Ice.openModule('Ice')
__name__ = 'Ice'

if 'ObjectFactory' not in _M_Ice.__dict__:
    _M_Ice.ObjectFactory = Ice.createTempClass()
    class ObjectFactory(object):
        """
        A factory for objects. Object factories are used when receiving "objects by value".
        An object factory must be implemented by the application writer and registered
        with the communicator.
        """
        def __init__(self):
            if Ice.getType(self) == _M_Ice.ObjectFactory:
                raise RuntimeError('Ice.ObjectFactory is an abstract class')

        def create(self, type):
            """
            Create a new object for a given object type. The type is the
            absolute Slice type id, i.e., the id relative to the
            unnamed top-level Slice module. For example, the absolute
            Slice type id for interfaces of type Bar in the module
            Foo is "::Foo::Bar".
            The leading "::" is required.
            Arguments:
            type -- The object type.
            Returns: The object created for the given type, or nil if the factory is unable to create the object.
            """
            raise NotImplementedError("method 'create' not implemented")

        def destroy(self):
            """
            Called when the factory is removed from the communicator, or if
            the communicator is destroyed.
            """
            raise NotImplementedError("method 'destroy' not implemented")

        def __str__(self):
            return IcePy.stringify(self, _M_Ice._t_ObjectFactory)

        __repr__ = __str__

    _M_Ice._t_ObjectFactory = IcePy.defineValue('::Ice::ObjectFactory', ObjectFactory, -1, (), False, True, None, ())
    ObjectFactory._ice_type = _M_Ice._t_ObjectFactory

    _M_Ice.ObjectFactory = ObjectFactory
    del ObjectFactory

# End of module Ice



#
# Copyright (c) ZeroC, Inc. All rights reserved.
#

# Always prefer setuptools over distutils
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

from distutils.extension import Extension
from distutils.command.build_ext import build_ext

import sys, os, shutil, fnmatch

platform = sys.platform
if platform[:6] == 'darwin':
    platform = 'darwin'
elif platform[:5] == 'linux':
    platform = 'linux'
elif platform[:7] == 'freebsd':
    platform = 'freebsd'

use_ice = False
if "--with-installed-ice" in sys.argv:
    use_ice = True
    sys.argv.remove("--with-installed-ice")

#
# Files from cpp/src/Slice that are required by IcePy
#
sliceSrcs = ["Checksum.cpp", "FileTracker.cpp", "Grammar.cpp", "MD5.cpp",
             "MD5I.cpp", "Parser.cpp", "Preprocessor.cpp", "Python.cpp",
             "PythonUtil.cpp", "Scanner.cpp", "SliceUtil.cpp", "StringLiteralUtil.cpp"]

#
# Sort out packages, package_dir and package_data from the lib dir. We include 'slice' in the
# package list and use package_data to ensure the Slice files are included. We also need to
# set include_package_data=True.
#
packages=['Glacier2', 'Ice', 'IceBox', 'IceGrid', 'IceMX', 'IcePatch2', 'IceStorm', 'slice']
if sys.version_info[:2] >= (3, 5):
    packages += ['Ice.Py3']
package_dir={'' : 'lib'}
package_data={'' : ['*.ice']}

extra_compile_args=[]
define_macros=[('ICE_PYPI', None)]
if use_ice:
    include_dirs=['src', 'src/ice/cpp/src']
else:
    include_dirs=['src', 'src/ice/cpp/include', 'src/ice/cpp/include/generated', 'src/ice/cpp/src']
    #
    # Define ICE_STATIC_LIBS to disable _API macros
    #
    define_macros+=[('ICE_STATIC_LIBS', None)]

if platform == 'darwin':
    if not 'ARCHFLAGS' in os.environ:
        os.environ['ARCHFLAGS'] = '-arch x86_64'
    # Make sure to use the SDK from Xcode (required for Sierra where old system headers can be used otherwise)
    os.environ['CC'] = 'xcrun -sdk macosx clang'
    os.environ['CXX'] = 'xcrun -sdk macosx clang++'
    extra_compile_args.append('-w')
    if use_ice:
        libraries = ["IceSSL", "IceLocatorDiscovery", "IceDiscovery", "Ice"]
        extra_link_args = []
    else:
        libraries=['iconv']
        extra_link_args = ['-framework','Security', '-framework','CoreFoundation']
elif platform == 'win32':
    extra_link_args = []
    libraries=[]
    define_macros.append(('WIN32_LEAN_AND_MEAN', None))
    define_macros.append(('ICE_BUILDING_SRC', None))
    define_macros.append(('ICE_BUILDING_ICE', None))
    define_macros.append(('ICE_BUILDING_ICESSL', None))
    define_macros.append(('ICE_BUILDING_ICE_LOCATOR_DISCOVERY', None))
    define_macros.append(('_WIN32_WINNT', '0x601'))
    include_dirs.append('src/ice/bzip2')
    extra_compile_args.append('/EHsc')
    extra_compile_args.append('/wd4018')
    extra_compile_args.append('/wd4146')
    extra_compile_args.append('/wd4244')
    extra_compile_args.append('/wd4250')
    extra_compile_args.append('/wd4251')
    extra_compile_args.append('/wd4267')
    extra_compile_args.append('/wd4275')
    extra_compile_args.append('/wd4996')
    extra_compile_args.append('/Zi')
    if sys.version_info[:2] >= (3, 6):
        extra_link_args.append('/DEBUG:FULL')
    else:
        extra_link_args.append('/DEBUG')
    libraries=['dbghelp', 'Shlwapi', 'rpcrt4','advapi32','Iphlpapi','secur32','crypt32','ws2_32']
else:
    #
    # TODO: Get rid of this hack to remove -Wstrict-prototypes from the compiler options
    # when http://bugs.python.org/issue1222585 is fixed. Note that this hack doesn't work
    # with recent distutils versions which no longer allow overriding OPT in the env.
    #
    from distutils.sysconfig import get_config_vars
    (opt,) = get_config_vars('OPT')
    os.environ['OPT'] = " ".join(flag for flag in opt.split() if flag != '-Wstrict-prototypes')

    extra_compile_args.append('-w')
    extra_link_args = []
    if use_ice:
        libraries = ["IceSSL", "IceLocatorDiscovery", "IceDiscovery", "Ice"]
    else:
        libraries=['ssl', 'crypto', 'bz2', 'rt']
        if platform is not 'freebsd':
            libraries.append('dl')

def filterName(path):
    d = os.path.dirname(path)
    b = os.path.basename(path)
    #
    # Always build mcpp sources
    #
    if "mcpp" in d:
        return True
    #
    # Always build Slice sources required by IcePy
    #
    elif os.path.normpath("ice/cpp/src/Slice") in d:
        return b in sliceSrcs
    if platform == "win32":
        #
        # SysLoggerI.cpp shouldn't be built under Windows.
        #
        if b == 'SysLoggerI.cpp':
            return False
        #
        # Don't build OpenSSL, SecureTransport and UWP sources on
        # Windows
        #
        if b.startswith("UWP") or b.startswith("OpenSSL") or b.startswith("SecureTransport"):
            return False
    else:
        #
        # Filter IceSSL sources that doesn't match current OS default
        # implementation
        #
        if (b.startswith("SChannel") or b.startswith("UWP") or (b.startswith("OpenSSL") and platform == "darwin") or
            (b.startswith("SecureTransport") and platform != "darwin")):
            return False

        #
        # Don't build Ice for C++ sources if using Ice system install (--with-installed-ice)
        #
        if use_ice and os.path.normpath("src/ice/") in d:
            return False
        #
        # Always use system bzip2 under Unix platforms.
        #
        elif d.find('bzip2') != -1:
            return False
    return True

# Gather the list of sources to compile.
sources = []
for root, dirnames, filenames in os.walk('src'):
  for filename in fnmatch.filter(filenames, '*.cpp'):
        n = os.path.normpath(os.path.join(root, filename))
        if filterName(n):
            sources.append(n)
  for filename in fnmatch.filter(filenames, '*.c'):
        n = os.path.normpath(os.path.join(root, filename))
        if filterName(n):
            sources.append(n)

with open('README.rst') as file:
    long_description = file.read()

class BuildExt(build_ext):
    def build_extensions(self):
        if platform == "win32":
            for e in self.extensions:
                e.extra_link_args.append('/PDB:{0}'.format(os.path.join(self.build_temp, "IcePy.pdb")))
        build_ext.build_extensions(self)

setup(
    name='zeroc-ice',

    version='3.7.6',

    description="Ice is a comprehensive RPC framework with support for Python, C++, .NET, Java, JavaScript and more.",

    long_description=long_description,

    # The project's main homepage.
    url='https://zeroc.com',

    # Author details
    author='ZeroC, Inc.',
    author_email='info@zeroc.com',

    # Choose your license
    license='GPL v2 with exceptions',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 5 - Production/Stable',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.0',
        'Programming Language :: Python :: 3.1',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],

    # What does your project relate to?
    keywords='RPC distributed systems development',

    packages = packages,
    package_dir = package_dir,
    package_data = package_data,
    include_package_data = True,
    exclude_package_data={'slice': ['IceDiscovery/*.ice', 'IceLocatorDiscovery/*.ice']},
    py_modules = ["slice2py"],
    cmdclass = {'build_ext': BuildExt },
    entry_points = {
        'console_scripts': ['slice2py=slice2py:main'],
    },

    ext_modules=[
        Extension('IcePy', sources,
          extra_link_args=extra_link_args,
          define_macros=define_macros,
          include_dirs=include_dirs,
          extra_compile_args=extra_compile_args,
          libraries=libraries)
        ]
)
